@@ -5,7 +5,7 @@
 # ENV['RAILS_ENV'] ||= 'production'
 
 # Specifies gem version of Rails to use when vendor/rails is not present
-RAILS_GEM_VERSION = '2.2.2' unless defined? RAILS_GEM_VERSION
+RAILS_GEM_VERSION = '2.3.2' unless defined? RAILS_GEM_VERSION
 
 # Bootstrap the Rails environment, frameworks, and default configuration
 require File.join(File.dirname(__FILE__), 'boot')
@@ -27,7 +27,7 @@ Rails::Initializer.run do |config|
   # config.gem "hpricot", :version => '0.6', :source => "http://code.whytheluckystiff.net"
   # config.gem "sqlite3-ruby", :lib => "sqlite3"
   # config.gem "aws-s3", :lib => "aws/s3"
-  config.gem "locale_rails", :version => "0.1.0"
+  config.gem "locale_rails", :version => "2.0.4"
   # We want to use rails-exception_notification. :<
   config.gem "jeremyevans-exception_notification",
              :lib => "exception_notification",
@@ -62,9 +62,18 @@ Rails::Initializer.run do |config|
   # If you change this key, all old sessions will become invalid!
   # Make sure the secret is at least 30 characters and all random, 
   # no regular words or you'll be exposed to dictionary attacks.
+  secret_file = File.join(RAILS_ROOT, "config", "secret")
+  unless File.exist?(secret_file)
+    require 'activesupport'
+    secret = ActiveSupport::SecureRandom.hex(64)
+    File.open(secret_file, "w") do |output|
+      output.puts(secret)
+    end
+  end
+  secret = File.read(secret_file).strip
   config.action_controller.session = {
     :session_key => '_milter-manager-admin_session',
-    :secret      => 'ba5f4e4f9159453750dffc73982b5b065196226c038f8eeb71bee99576646b5588ff7890f36356fcebce2f600e30a6d61d13afb1969c954e4f4b0b889128e24c'
+    :secret      => secret,
   }
 
   # Use the database for sessions instead of the cookie-based default,
