@@ -30,7 +30,7 @@ require 'milter/compatible'
 require 'milter/manager/child-context'
 
 require 'milter/manager/debian-init-detector'
-require 'milter/manager/redhat-init-detector'
+require 'milter/manager/redhat-detector'
 require 'milter/manager/freebsd-rc-detector'
 require 'milter/manager/pkgsrc-rc-detector'
 
@@ -55,6 +55,11 @@ module Milter::Manager
       result << "manager.remove_unix_socket_on_close = #{remove_manager_unix_socket_on_close?}\n"
       result << "manager.daemon = #{daemon?}\n"
       result << "manager.pid_file = #{pid_file.inspect}\n"
+      _maintenance_interval = maintenance_interval
+      _maintenance_interval = nil if _maintenance_interval.zero?
+      result << "manager.maintenance_interval = #{_maintenance_interval.inspect}\n"
+      result << "manager.suspend_time_on_unacceptable = #{suspend_time_on_unacceptable.inspect}\n"
+      result << "manager.custom_configuration_directory = #{custom_configuration_directory.inspect}\n"
       result << "\n"
 
       result << "controller.connection_spec = #{controller_connection_spec.inspect}\n"
@@ -76,6 +81,8 @@ module Milter::Manager
         result << "  milter.connection_spec = #{egg.connection_spec.inspect}\n"
         result << "  milter.description = #{egg.description.inspect}\n"
         result << "  milter.enabled = #{egg.enabled?}\n"
+        fallback_status = egg.fallback_status.nick
+        result << "  milter.fallback_status = #{fallback_status.inspect}\n"
         condition_names = egg.applicable_conditions.collect do |condition|
           condition.name
         end
@@ -137,6 +144,7 @@ module Milter::Manager
         guard do
           new(configuration).load_configuration(file)
         end
+        GC.start
       end
 
       def load_custom(configuration, file)
@@ -144,6 +152,7 @@ module Milter::Manager
         guard do
           new(configuration).load_custom_configuration(file)
         end
+        GC.start
       end
 
       def guard(fallback_value=nil)
@@ -337,6 +346,9 @@ module Milter::Manager
                 milter.command = @egg_config["command"]
                 milter.command_options = @egg_config["command_options"]
               end
+              if @egg_config.has_key?("fallback_status")
+                milter.fallback_status = @egg_config["fallback_status"]
+              end
             end
             @egg_config = nil
           when "milter_applicable_condition"
@@ -397,7 +409,8 @@ module Milter::Manager
           when :milter
             available_locals = ["name", "description",
                                 "enabled", "connection_spec",
-                                "command", "command_options"]
+                                "command", "command_options",
+                                "fallback_status"]
             case local
             when "applicable_conditions"
               @egg_config["applicable_conditions"] = []
@@ -610,6 +623,32 @@ module Milter::Manager
       def pid_file
         @configuration.pid_file
       end
+
+      def maintenance_interval=(n_sessions)
+        n_sessions ||= 0
+        @configuration.maintenance_interval = n_sessions
+      end
+
+      def maintenance_interval
+        @configuration.maintenance_interval
+      end
+
+      def suspend_time_on_unacceptable=(seconds)
+        seconds ||= Milter::Client::DEFAULT_SUSPEND_TIME_ON_UNACCEPTABLE
+        @configuration.suspend_time_on_unacceptable = seconds
+      end
+
+      def suspend_time_on_unacceptable
+        @configuration.suspend_time_on_unacceptable
+      end
+
+      def custom_configuration_directory=(directory)
+        @configuration.custom_configuration_directory = directory
+      end
+
+      def custom_configuration_directory
+        @configuration.custom_configuration_directory
+      end
     end
 
     class EggConfiguration
@@ -636,7 +675,11 @@ module Milter::Manager
       def applicable_conditions=(condition_names)
         configuration = @loader.configuration
         conditions = (condition_names || []).collect do |name|
-          condition = configuration.find_applicable_condition(name)
+          if name.is_a?(Milter::Manager::ApplicableCondition)
+            condition = name
+          else
+            condition = configuration.find_applicable_condition(name)
+          end
           if condition.nil?
             available_conditions = configuration.applicable_conditions
             available_condition_names =
@@ -662,6 +705,32 @@ module Milter::Manager
         @egg.command_options = options
       end
 
+      def fallback_status
+        status = @egg.fallback_status
+        status.nick
+      end
+
+      def fallback_status=(status)
+        available_values = {
+          "accept" => Milter::Status::ACCEPT,
+          "reject" => Milter::Status::REJECT,
+          "temporary-failure" => Milter::Status::TEMPORARY_FAILURE,
+          "discard" => Milter::Status::DISCARD,
+        }
+        if status.respond_to?(:nick)
+          normalized_status = status.nick
+        else
+          normalized_status = status.to_s.downcase.gsub(/_/, '-')
+        end
+        value = available_values[normalized_status]
+        if value.nil?
+          raise InvalidValue.new("milter.fallback_status",
+                                 available_values.keys,
+                                 status)
+        end
+        @egg.fallback_status = value
+      end
+
       def method_missing(name, *args, &block)
         @egg.send(name, *args, &block)
       end
@@ -677,9 +746,14 @@ module Milter::Manager
         @condition = Milter::Manager::ApplicableCondition.new(name)
         @loader = loader
         @connect_stoppers = []
+        @helo_stoppers = []
         @envelope_from_stoppers = []
         @envelope_recipient_stoppers = []
+        @data_stoppers = []
         @header_stoppers = []
+        @end_of_header_stoppers = []
+        @body_stoppers = []
+        @end_of_message_stoppers = []
 
         exist_condition = @loader.configuration.find_applicable_condition(name)
         @condition.merge(exist_condition) if exist_condition
@@ -689,6 +763,10 @@ module Milter::Manager
         @connect_stoppers << block
       end
 
+      def define_helo_stopper(&block)
+        @helo_stoppers << block
+      end
+
       def define_envelope_from_stopper(&block)
         @envelope_from_stoppers << block
       end
@@ -697,15 +775,37 @@ module Milter::Manager
         @envelope_recipient_stoppers << block
       end
 
+      def define_data_stopper(&block)
+        @data_stoppers << block
+      end
+
       def define_header_stopper(&block)
         @header_stoppers << block
       end
 
+      def define_end_of_header_stopper(&block)
+        @end_of_header_stoppers << block
+      end
+
+      def define_body_stopper(&block)
+        @body_stoppers << block
+      end
+
+      def define_end_of_message_stopper(&block)
+        @end_of_message_stoppers << block
+      end
+
       def have_stopper?
         [@connect_stoppers,
+         @helo_stoppers,
          @envelope_from_stoppers,
          @envelope_recipient_stoppers,
-         @header_stoppers].any? do |stoppers|
+         @data_stoppers,
+         @header_stoppers,
+         @end_of_header_stoppers,
+         @body_stoppers,
+         @end_of_message_stoppers,
+        ].any? do |stoppers|
           not stoppers.empty?
         end
       end
@@ -736,6 +836,17 @@ module Milter::Manager
             end
           end
 
+          unless @helo_stoppers.empty?
+            child.signal_connect("stop-on-helo") do |_child, fqdn|
+              context = ChildContext.new(child, children)
+              @helo_stoppers.any? do |stopper|
+                ConfigurationLoader.guard(false) do
+                  stopper.call(context, fqdn)
+                end
+              end
+            end
+          end
+
           unless @envelope_from_stoppers.empty?
             child.signal_connect("stop-on-envelope-from") do |_child, from|
               context = ChildContext.new(child, children)
@@ -758,6 +869,17 @@ module Milter::Manager
             end
           end
 
+          unless @data_stoppers.empty?
+            child.signal_connect("stop-on-data") do |_child|
+              context = ChildContext.new(child, children)
+              @data_stoppers.any? do |stopper|
+                ConfigurationLoader.guard(false) do
+                  stopper.call(context)
+                end
+              end
+            end
+          end
+
           unless @header_stoppers.empty?
             child.signal_connect("stop-on-header") do |_child, name, value|
               context = ChildContext.new(child, children)
@@ -768,6 +890,39 @@ module Milter::Manager
               end
             end
           end
+
+          unless @end_of_header_stoppers.empty?
+            child.signal_connect("stop-on-end-of-header") do |_child|
+              context = ChildContext.new(child, children)
+              @end_of_header_stoppers.any? do |stopper|
+                ConfigurationLoader.guard(false) do
+                  stopper.call(context)
+                end
+              end
+            end
+          end
+
+          unless @body_stoppers.empty?
+            child.signal_connect("stop-on-body") do |_child, chunk|
+              context = ChildContext.new(child, children)
+              @body_stoppers.any? do |stopper|
+                ConfigurationLoader.guard(false) do
+                  stopper.call(context, name, chunk)
+                end
+              end
+            end
+          end
+
+          unless @end_of_message_stoppers.empty?
+            child.signal_connect("stop-on-end-of-message") do |_child, chunk|
+              context = ChildContext.new(child, children)
+              @end_of_message_stoppers.any? do |stopper|
+                ConfigurationLoader.guard(false) do
+                  stopper.call(context)
+                end
+              end
+            end
+          end
         end
       end
     end
