@@ -118,6 +118,41 @@ class TestConfigurationLoader < Test::Unit::TestCase
     assert_equal(@configuration.pid_file, @loader.manager.pid_file)
   end
 
+  def test_manager_maintenance_interval
+    assert_equal(100, @configuration.maintenance_interval)
+
+    @loader.manager.maintenance_interval = nil
+    assert_equal(0, @configuration.maintenance_interval)
+
+    @loader.manager.maintenance_interval = 0
+    assert_equal(0, @configuration.maintenance_interval)
+
+    @loader.manager.maintenance_interval = 10
+    assert_equal(10, @configuration.maintenance_interval)
+  end
+
+  def test_manager_suspend_time_on_unacceptable
+    assert_equal(5, @configuration.suspend_time_on_unacceptable)
+
+    @loader.manager.suspend_time_on_unacceptable = 0
+    assert_equal(0, @configuration.suspend_time_on_unacceptable)
+
+    @loader.manager.suspend_time_on_unacceptable = nil
+    assert_equal(5, @configuration.suspend_time_on_unacceptable)
+
+    @loader.manager.suspend_time_on_unacceptable = 10
+    assert_equal(10, @configuration.suspend_time_on_unacceptable)
+  end
+
+  def test_manager_configuration_directory
+    assert_nil(@configuration.custom_configuration_directory)
+    @loader.manager.custom_configuration_directory = "/tmp/milter-manager/"
+    assert_equal("/tmp/milter-manager/",
+                 @configuration.custom_configuration_directory)
+    assert_equal(@configuration.custom_configuration_directory,
+                 @loader.manager.custom_configuration_directory)
+  end
+
   def test_to_xml
     assert_equal(<<-EOX, @configuration.to_xml)
 <configuration>
@@ -161,6 +196,7 @@ EOX
     <milter>
       <name>child-milter</name>
       <enabled>true</enabled>
+      <fallback-status>accept</fallback-status>
       <connection-spec>unix:/tmp/socket</connection-spec>
       <applicable-conditions>
         <applicable-condition>remote-network</applicable-condition>
@@ -280,6 +316,41 @@ EOX
                  end)
   end
 
+  def test_define_milter_add_applicable_condition
+    @loader.define_applicable_condition("S25R") do |condition|
+      condition.description = "Selective SMTP Rejection."
+    end
+    @loader.define_applicable_condition("Remote Network") do |condition|
+      condition.description = "Only works for remote network."
+    end
+
+    @loader.define_milter("milter1") do |milter|
+      milter.applicable_conditions = ["S25R"]
+      milter.applicable_conditions += ["Remote Network"]
+    end
+    assert_equal([["milter1", ["S25R", "Remote Network"]]],
+                 @configuration.eggs.collect do |egg|
+                   [egg.name,
+                    egg.applicable_conditions.collect {|cond| cond.name}]
+                 end)
+  end
+
+  def test_define_milter_fallback_status
+    @loader.define_milter("milter") do |milter|
+      assert_equal("accept", milter.fallback_status)
+      milter.fallback_status = "temporary_failure"
+      assert_equal("temporary-failure", milter.fallback_status)
+    end
+  end
+
+  def test_define_milter_invalid_fallback_status
+    @loader.define_milter("milter") do |milter|
+      assert_raise(Milter::Manager::ConfigurationLoader::InvalidValue) do
+        milter.fallback_status = "default"
+      end
+    end
+  end
+
   def test_define_applicable_condition_again
     @loader.define_applicable_condition("S25R") do |condition|
       condition.description = "Selective SMTP Rejection."
