@@ -33,6 +33,18 @@ class TestConfiguration < Test::Unit::TestCase
     assert_nil(@configuration.manager_connection_spec)
   end
 
+  def test_maintenance_interval
+    assert_equal(100, @configuration.maintenance_interval)
+    @configuration.maintenance_interval = 0
+    assert_equal(0, @configuration.maintenance_interval)
+  end
+
+  def test_suspend_time_on_unacceptable
+    assert_equal(5, @configuration.suspend_time_on_unacceptable)
+    @configuration.suspend_time_on_unacceptable = 10
+    assert_equal(10, @configuration.suspend_time_on_unacceptable)
+  end
+
   def test_package
     @configuration.package_platform = "pkgsrc"
     assert_equal("pkgsrc", @configuration.package_platform)
@@ -128,6 +140,7 @@ class TestConfiguration < Test::Unit::TestCase
                   "    <milter>",
                   "      <name>#{name}</name>",
                   "      <enabled>true</enabled>",
+                  "      <fallback-status>accept</fallback-status>",
                   "      <connection-spec>#{spec}</connection-spec>",
                   "    </milter>",
                   "  </milters>",
@@ -143,6 +156,7 @@ class TestConfiguration < Test::Unit::TestCase
                   "    <milter>",
                   "      <name>#{name}</name>",
                   "      <enabled>true</enabled>",
+                  "      <fallback-status>accept</fallback-status>",
                   "      <connection-spec>#{spec}</connection-spec>",
                   "    </milter>",
                   "  </milters>",
@@ -184,6 +198,9 @@ manager.remove_unix_socket_on_create = true
 manager.remove_unix_socket_on_close = true
 manager.daemon = false
 manager.pid_file = nil
+manager.maintenance_interval = 100
+manager.suspend_time_on_unacceptable = 5
+manager.custom_configuration_directory = nil
 
 controller.connection_spec = nil
 controller.unix_socket_mode = 0660
@@ -230,6 +247,9 @@ manager.remove_unix_socket_on_create = true
 manager.remove_unix_socket_on_close = true
 manager.daemon = false
 manager.pid_file = nil
+manager.maintenance_interval = 100
+manager.suspend_time_on_unacceptable = 5
+manager.custom_configuration_directory = nil
 
 controller.connection_spec = nil
 controller.unix_socket_mode = 0660
@@ -245,6 +265,7 @@ define_milter("milter1") do |milter|
   milter.connection_spec = "unix:/tmp/xxx"
   milter.description = "The first milter"
   milter.enabled = true
+  milter.fallback_status = "accept"
   milter.applicable_conditions = ["S25R"]
   milter.command = nil
   milter.command_options = nil
@@ -259,6 +280,7 @@ define_milter("milter2") do |milter|
   milter.connection_spec = "inet:2929"
   milter.description = "The second milter"
   milter.enabled = false
+  milter.fallback_status = "accept"
   milter.applicable_conditions = []
   milter.command = nil
   milter.command_options = nil
