@@ -16,7 +16,7 @@
 # along with this library.  If not, see <http://www.gnu.org/licenses/>.
 
 class TestRedHatInitDetector < Test::Unit::TestCase
-  include MilterTestUtils
+  include MilterDetectorTestUtils
 
   def setup
     @configuration = Milter::Manager::Configuration.new
@@ -24,7 +24,8 @@ class TestRedHatInitDetector < Test::Unit::TestCase
     @tmp_dir = Pathname(File.dirname(__FILE__)) + ".." + "tmp"
     @init_base_dir = @tmp_dir
     @init_d = @init_base_dir + "init.d"
-    @sysconfig_dir = @tmp_dir + "sysconfig"
+    @etc_dir = @tmp_dir + "etc"
+    @sysconfig_dir = @etc_dir + "sysconfig"
     @init_d.mkpath
     @sysconfig_dir.mkpath
   end
@@ -60,7 +61,7 @@ class TestRedHatInitDetector < Test::Unit::TestCase
                    "prog" => "milter-manager",
                    "milter_manager" => "/usr/sbin/milter-manager",
                    "USER" => "milter-manager",
-                   "GROUP" => "smmsp",
+                   "GROUP" => "milter-manager",
                    "SOCKET_GROUP" => "smmsp",
                    "PIDFILE" => "/var/run/milter-manager/milter-manager.pid",
                    "CONNECTION_SPEC" => "unix:/var/run/milter-manager/milter-manager.sock",
@@ -71,7 +72,7 @@ class TestRedHatInitDetector < Test::Unit::TestCase
                                      "unix:/var/run/milter-manager/milter-manager.sock",
                                      "--user-name",
                                      "milter-manager",
-                                     "--group-name", "smmsp",
+                                     "--group-name", "milter-manager",
                                      "--socket-group-name", "smmsp",
                                     ].join(" ") + " ",
                    "OPTION_ARGS" => "",
@@ -123,7 +124,7 @@ EOC
                    "prog" => "milter-manager",
                    "milter_manager" => "/usr/sbin/milter-manager",
                    "USER" => "milter-manager",
-                   "GROUP" => "smmsp",
+                   "GROUP" => "milter-manager",
                    "SOCKET_GROUP" => "smmsp",
                    "PIDFILE" => "/var/run/milter-manager/milter-manager.pid",
                    "CONNECTION_SPEC" => "inet:10025@localhost",
@@ -134,7 +135,7 @@ EOC
                                      "unix:/var/run/milter-manager/milter-manager.sock",
                                      "--user-name",
                                      "milter-manager",
-                                     "--group-name", "smmsp",
+                                     "--group-name", "milter-manager",
                                      "--socket-group-name", "smmsp",
                                     ].join(" ") + " ",
                    "OPTION_ARGS" => "",
@@ -221,9 +222,9 @@ EOC
                   "inet:10025@localhost"]])
   end
 
-  def test_apply_clamav_milter_style
+  def test_apply_clamav_milter_before_0_95_style
     (@init_d + "clamav-milter").open("w") do |file|
-      file << clamav_milter_init_header
+      file << clamav_milter_before_0_95_init_header
     end
 
     detector = redhat_init_detector("clamav-milter")
@@ -239,9 +240,9 @@ EOC
   end
 
 
-  def test_apply_clamav_milter_style_with_config
+  def test_apply_clamav_milter_before_0_95_style_with_config
     (@init_d + "clamav-milter").open("w") do |file|
-      file << clamav_milter_init_header
+      file << clamav_milter_before_0_95_init_header
     end
     create_rc_files("clamav-milter")
     (@sysconfig_dir + "clamav-milter").open("w") do |file|
@@ -263,6 +264,47 @@ EOC
                   "inet:11121@[127.0.0.1]"]])
   end
 
+  def test_apply_clamav_milter_style
+    (@init_d + "clamav-milter").open("w") do |file|
+      file << clamav_milter_init_header
+    end
+    create_rc_files("clamav-milter")
+
+    clamav_milter_conf = @tmp_dir + "clamav-milter.conf"
+    (@sysconfig_dir + "clamav-milter").open("w") do |file|
+      file << <<-EOC
+### Simple config file for clamav-milter, you should
+### read the documentation and tweak it as you wish.
+
+CLAMAV_FLAGS="--config-file=#{clamav_milter_conf}"
+EOC
+    end
+    clamav_milter_conf.open("w") do |file|
+      file << <<-'EOC'
+# Define the interface through which we communicate with sendmail
+# This option is mandatory! Possible formats are:
+# [[unix|local]:]/path/to/file - to specify a unix domain socket
+# inet:port@[hostname|ip-address] - to specify an ipv4 socket
+# inet6:port@[hostname|ip-address] - to specify an ipv6 socket
+#
+# Default: no default
+MilterSocket unix:/var/clamav/clmilter.socket
+#MilterSocket inet:7357
+EOC
+    end
+
+    detector = redhat_init_detector("clamav-milter")
+    detector.detect
+    detector.apply(@loader)
+    assert_eggs([["clamav-milter",
+                  "clamav-milter is a daemon which hooks into sendmail " +
+                  "and routes email messages to clamav.",
+                  true,
+                  (@init_d + "clamav-milter").to_s,
+                  "start",
+                  "unix:/var/clamav/clmilter.socket"]])
+  end
+
 
   def test_apply_milter_greylist_style
     (@init_d + "milter-greylist").open("w") do |file|
@@ -282,7 +324,7 @@ EOC
                   "unix:/var/milter-greylist/milter-greylist.sock"]])
   end
 
-  def test_apply_milter_greylist_style_with_config
+  def test_apply_milter_greylist_style_with_sysconfig
     (@init_d + "milter-greylist").open("w") do |file|
       file << milter_greylist_init_header
     end
@@ -305,6 +347,84 @@ EOC
                   "inet:10025@localhost"]])
   end
 
+  def test_apply_milter_greylist_style_with_config
+    etc_mail_dir = @etc_dir + "mail"
+    etc_mail_dir.mkpath
+    greylist_conf = etc_mail_dir + "greylist.conf"
+    (@init_d + "milter-greylist").open("w") do |file|
+      file << milter_greylist_init_header(greylist_conf)
+    end
+    create_rc_files("milter-greylist")
+
+    greylist_conf.open("w") do |file|
+      file << <<-'EOC'
+#
+# Simple greylisting config file using the new features
+# See greylist2.conf for a more detailed list of available options
+#
+# $Id: greylist.conf,v 1.45.2.1 2009/02/12 22:39:01 manu Exp $
+#
+
+#pidfile "/var/run/milter-greylist.pid"
+socket "inet:10025@[127.0.0.1]"
+dumpfile "/var/lib/milter-greylist/db/greylist.db" 600
+geoipdb "/usr/share/GeoIP/GeoIP.dat"
+dumpfreq 1
+user "grmilter"
+EOC
+    end
+
+    detector = redhat_init_detector("milter-greylist")
+    detector.detect
+    detector.apply(@loader)
+    assert_equal("milter-greylist", detector.name)
+    assert_eggs([["milter-greylist",
+                  "Milter Greylist Daemon",
+                  true,
+                  (@init_d + "milter-greylist").to_s,
+                  "start",
+                  "unix:/var/milter-greylist/milter-greylist.sock"]])
+  end
+
+  def test_apply_milter_greylist_fedora_style
+    etc_mail_dir = @etc_dir + "mail"
+    etc_mail_dir.mkpath
+    greylist_conf = etc_mail_dir + "greylist.conf"
+    (@init_d + "milter-greylist").open("w") do |file|
+      file << milter_greylist_init_header_fedora(greylist_conf)
+    end
+    create_rc_files("milter-greylist")
+
+    greylist_conf.open("w") do |file|
+      file << <<-'EOC'
+#
+# Simple greylisting config file using the new features
+# See greylist2.conf for a more detailed list of available options
+#
+# $Id: greylist.conf,v 1.45.2.1 2009/02/12 22:39:01 manu Exp $
+#
+
+#pidfile "/var/run/milter-greylist.pid"
+socket "/var/run/milter-greylist/milter-greylist.sock"
+dumpfile "/var/lib/milter-greylist/db/greylist.db" 600
+geoipdb "/usr/share/GeoIP/GeoIP.dat"
+dumpfreq 1
+user "grmilter"
+EOC
+    end
+
+    detector = redhat_init_detector("milter-greylist")
+    detector.detect
+    detector.apply(@loader)
+    assert_equal("milter-greylist", detector.name)
+    assert_eggs([["milter-greylist",
+                  "Milter Greylist Daemon",
+                  true,
+                  (@init_d + "milter-greylist").to_s,
+                  "start",
+                  "unix:/var/run/milter-greylist/milter-greylist.sock"]])
+  end
+
   def test_apply_spamass_milter_style
     (@init_d + "spamass-milter").open("w") do |file|
       file << spamass_milter_init_header
@@ -323,19 +443,52 @@ EOC
                   "unix:/var/run/spamass.sock"]])
   end
 
+
+  def test_apply_enma_style
+    enma_conf = @tmp_dir + "enma.conf"
+    (@init_d + "enma").open("w") do |file|
+      file << enma_init_header(enma_conf.to_s)
+    end
+
+    enma_conf.open("w") do |conf|
+      conf << <<-EOC
+## Milter ##
+milter.socket:  inet:10025@127.0.0.1
+milter.user:    daemon
+milter.pidfile: /var/run/enma/enma.pid
+milter.chdir:   /var/tmp
+milter.timeout: 7210
+milter.loglevel:   0
+milter.sendmail813: false
+milter.postfix: false
+EOC
+    end
+
+    detector = redhat_init_detector("enma")
+    detector.detect
+    detector.apply(@loader)
+    assert_equal("enma", detector.name)
+    assert_eggs([["enma",
+                  "A milter program for domain authentication technologies",
+                  false,
+                  (@init_d + "enma").to_s,
+                  "start",
+                  "inet:10025@127.0.0.1"]])
+  end
+
   private
   def redhat_init_detector(name)
     detector = Milter::Manager::RedHatInitDetector.new(@configuration, name)
 
     _init_base_dir = @init_base_dir
-    _sysconfig_dir = @sysconfig_dir
+    _etc_dir = @etc_dir
     singleton_object = class << detector; self; end
     singleton_object.send(:define_method, :init_base_dir) do
       _init_base_dir.to_s
     end
 
-    singleton_object.send(:define_method, :sysconfig_dir) do
-      _sysconfig_dir.to_s
+    singleton_object.send(:define_method, :etc_dir) do
+      _etc_dir.to_s
     end
 
     detector
@@ -395,6 +548,33 @@ SYSCONFIG="/etc/sysconfig/amavisd"
 EOS
   end
 
+  def clamav_milter_before_0_95_init_header
+    <<-'EOS'
+#!/bin/sh
+#
+# Startup script for the Clamav Milter Daemon
+#
+# chkconfig: 2345 77 23
+# description: clamav-milter is a daemon which hooks into sendmail \
+#              and routes email messages to clamav.
+# processname: clamav-milter
+# pidfile: /var/run/clamav/clamav-milter.pid
+# config: /etc/sysconfig/clamav-milter
+
+# Source function library.
+. /etc/rc.d/init.d/functions
+
+# Source networking configuration.
+. /etc/sysconfig/network
+
+[ -x /usr/sbin/clamav-milter ] || exit 0
+
+# Local clamav-milter config
+CLAMAV_FLAGS=
+test -f /etc/sysconfig/clamav-milter && . /etc/sysconfig/clamav-milter
+EOS
+  end
+
   def clamav_milter_init_header
     <<-'EOS'
 #!/bin/sh
@@ -422,8 +602,8 @@ test -f /etc/sysconfig/clamav-milter && . /etc/sysconfig/clamav-milter
 EOS
   end
 
-  def milter_greylist_init_header
-    <<-'EOS'
+  def milter_greylist_init_header(conf_file=nil)
+    <<-EOS
 #!/bin/sh
 # $Id: rc-redhat.sh.in,v 1.7 2006/08/20 05:20:51 manu Exp $
 #  init file for milter-greylist
@@ -432,7 +612,7 @@ EOS
 # description: Milter Greylist Daemon
 #
 # processname: /usr/bin/milter-greylist
-# config: /etc/mail/greylist.conf
+# config: #{conf_file || '/etc/mail/greylist.conf'}
 # pidfile: /var/milter-greylist/milter-greylist.pid
 
 # source function library
@@ -451,6 +631,33 @@ prog="Milter-Greylist"
 EOS
   end
 
+  def milter_greylist_init_header_fedora(conf_file)
+    <<-EOS
+#!/bin/sh
+# $Id: rc-redhat.sh.in,v 1.7 2006/08/20 05:20:51 manu Exp $
+#  init file for milter-greylist
+#
+# chkconfig: - 79 21
+# description: Milter Greylist Daemon
+#
+# processname: /usr/sbin/milter-greylist
+# config: #{conf_file}
+# pidfile: /var/run/milter-greylist.pid
+
+# source function library
+. /etc/init.d/functions
+
+pidfile="/var/run/milter-greylist.pid"
+OPTIONS="-P $pidfile"
+if [ -f /etc/sysconfig/milter-greylist ]
+then
+    . /etc/sysconfig/milter-greylist
+fi
+RETVAL=0
+prog="Milter-Greylist"
+EOS
+  end
+
   def spamass_milter_init_header
     <<-'EOS'
 #!/bin/bash
@@ -487,15 +694,31 @@ desc="Spamassassin sendmail milter"
 EOS
   end
 
-  def assert_eggs(expected_eggs)
-    egg_info = @configuration.eggs.collect do |egg|
-      [egg.name,
-       egg.description,
-       egg.enabled?,
-       egg.command,
-       egg.command_options,
-       egg.connection_spec]
-    end
-    assert_equal(expected_eggs, egg_info)
+  def enma_init_header(enma_conf)
+    <<-EOS
+#!/bin/bash
+#
+# Copyright (c) 2008-2009 Internet Initiative Japan Inc. All rights reserved.
+#
+# The terms and conditions of the accompanying program
+# shall be provided separately by Internet Initiative Japan Inc.
+# Any use, reproduction or distribution of the program are permitted
+# provided that you agree to be bound to such terms and conditions.
+#
+# $Id: rc.enma-centos 882 2009-04-02 01:02:28Z takahiko $
+# 
+# chkconfig: 345 79 31
+# description: A milter program for domain authentication technologies
+
+# source function library
+. /etc/init.d/functions
+
+RETVAL=0
+prog=enma
+
+ENMA=/usr/libexec/enma
+CONF_FILE=#{enma_conf}
+LOCK_FILE=/var/lock/subsys/enma
+EOS
   end
 end
