@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -91,6 +91,7 @@ G_BEGIN_DECLS
  *     GError *error = NULL;
  *
  *     milter_init();
+ *     milter_client_init();
  *
  *     client = milter_client_new();
  *     if (!milter_client_set_connection_spec(client, spec, &error)) {
@@ -102,6 +103,7 @@ G_BEGIN_DECLS
  *                      G_CALLBACK(cb_connection_established), NULL);
  *     milter_client_main(client);
  *
+ *     milter_client_quit();
  *     milter_quit();
  *
  *     return success ? EXIT_SUCCESS : EXIT_FAILURE;
@@ -131,6 +133,30 @@ G_BEGIN_DECLS
  * is not bottleneck. :-|</rd>
  */
 
+/**
+ * milter_client_init:
+ *
+ * Call this function before using any other the milter-client
+ * library functions.
+ */
+void milter_client_init (void);
+
+/**
+ * milter_client_quit:
+ *
+ * Call this function after the milter-client library use.
+ */
+void milter_client_quit (void);
+
+/**
+ * MILTER_CLIENT_DEFAULT_SUSPEND_TIME_ON_UNACCEPTABLE:
+ *
+ * The suspend time on unacceptable situation. See
+ * milter_client_get_suspend_time_on_unacceptable() for more
+ * details of the situation.
+ */
+#define MILTER_CLIENT_DEFAULT_SUSPEND_TIME_ON_UNACCEPTABLE 5
+
 /**
  * MILTER_CLIENT_ERROR:
  *
@@ -192,6 +218,11 @@ struct _MilterClientClass
     const gchar *(*get_unix_socket_group) (MilterClient *client);
     gboolean (*is_remove_unix_socket_on_close)
                                           (MilterClient *client);
+    guint  (*get_suspend_time_on_unacceptable)
+                                          (MilterClient *client);
+    void   (*set_suspend_time_on_unacceptable)
+                                          (MilterClient *client,
+                                           guint         suspend_time);
 };
 
 GQuark               milter_client_error_quark       (void);
@@ -403,13 +434,43 @@ gboolean             milter_client_get_default_remove_unix_socket_on_close
  * @remove: %TRUE if removing UNIX domain socket after the
  *          socket is closed.
  *
- * Gets default value whether removing UNIX domain socket
+ * Sets default value whether removing UNIX domain socket
  * after the socket is closed.
  */
 void                 milter_client_set_default_remove_unix_socket_on_close
                                                      (MilterClient  *client,
                                                       gboolean       remove);
 
+/**
+ * milter_client_get_suspend_time_on_unacceptable:
+ * @client: a %MilterClient.
+ *
+ * Gets suspend time on unacceptable situation. The
+ * situation is occurred when many SMTP clients connect to
+ * SMTP server. The situation means that your mail system
+ * is overflowed. You need to take some
+ * measures. e.g. Reducing number of milters, running
+ * milters on other machines, upgrade machine spec, growing
+ * number of max file descriptors and so on.
+ *
+ * Returns: suspend time in seconds.
+ */
+guint                milter_client_get_suspend_time_on_unacceptable
+                                                     (MilterClient  *client);
+
+/**
+ * milter_client_get_suspend_time_on_unacceptable:
+ * @client: a %MilterClient.
+ * @suspend_time: suspend time in seconds.
+ *
+ * Sets suspend time on unacceptable situation. See
+ * milter_client_get_suspend_time_on_unacceptable() for more
+ * details of the situation.
+ */
+void                 milter_client_set_suspend_time_on_unacceptable
+                                                     (MilterClient  *client,
+                                                      guint          suspend_time);
+
 /**
  * milter_client_main:
  * @client: a %MilterClient.
