@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -64,7 +64,11 @@ G_BEGIN_DECLS
  * : ((<xxfi_eom|URL:https://www.milter.org/developers/api/xxfi_eom>))
  *    #MilterClientContext::end-of-message
  * : ((<xxfi_abort|URL:https://www.milter.org/developers/api/xxfi_abort>))
- *    #MilterClientContext::abort
+ *    #MilterClientContext::abort.
+ *
+ *    NOTE: You will need to check whether the current state is
+ *    message processing or not. You can use
+ *    %MILTER_CLIENT_CONTEXT_STATE_IN_MESSAGE_PROCESSING for it.
  * : ((<xxfi_close|URL:https://www.milter.org/developers/api/xxfi_close>))
  *    #MilterFinishedEmittable::finished
  * </rd>
@@ -145,14 +149,17 @@ G_BEGIN_DECLS
  * }
  * 
  * static MilterStatus
- * cb_end_of_message (MilterClientContext *context, gpointer user_data)
+ * cb_end_of_message (MilterClientContext *context,
+ *                    const gchar *chunk, gsize length,
+ *                    gpointer user_data)
  * {
  *     g_print("end-of-message\n");
  *     return MILTER_STATUS_CONTINUE;
  * }
  * 
  * static MilterStatus
- * cb_abort (MilterClientContext *context, gpointer user_data)
+ * cb_abort (MilterClientContext *context, MilterClientContextState state,
+ *           gpointer user_data)
  * {
  *     g_print("abort\n");
  *     return MILTER_STATUS_CONTINUE;
@@ -318,6 +325,10 @@ typedef enum
     MILTER_CLIENT_CONTEXT_STATE_FINISHED
 } MilterClientContextState;
 
+#define MILTER_CLIENT_CONTEXT_STATE_IN_MESSAGE_PROCESSING(state)        \
+    (MILTER_CLIENT_CONTEXT_STATE_ENVELOPE_FROM <= (state) &&            \
+     (state) < MILTER_CLIENT_CONTEXT_STATE_END_OF_MESSAGE)
+
 typedef struct _MilterClientContext         MilterClientContext;
 typedef struct _MilterClientContextClass    MilterClientContextClass;
 
@@ -381,11 +392,14 @@ struct _MilterClientContextClass
                                         gsize                size);
     void         (*body_response)      (MilterClientContext *context,
                                         MilterStatus         status);
-    MilterStatus (*end_of_message)     (MilterClientContext *context);
+    MilterStatus (*end_of_message)     (MilterClientContext *context,
+                                        const gchar         *chunk,
+                                        gsize                size);
     void         (*end_of_message_response)
                                        (MilterClientContext *context,
                                         MilterStatus         status);
-    MilterStatus (*abort)              (MilterClientContext *context);
+    MilterStatus (*abort)              (MilterClientContext *context,
+                                        MilterClientContextState state);
     void         (*abort_response)     (MilterClientContext *context,
                                         MilterStatus         status);
     void         (*timeout)            (MilterClientContext *context);
@@ -753,6 +767,38 @@ void                 milter_client_context_set_state         (MilterClientContex
  */
 MilterClientContextState milter_client_context_get_state     (MilterClientContext *context);
 
+/**
+ * milter_client_context_get_last_state:
+ * @context: a %MilterClientContext.
+ *
+ * Gets the last state. It's one of start, negotiate,
+ * connect, helo, envelope-from, envelope-recipient, data,
+ * unknown, header, end-of-header, body and end-of-message.
+ *
+ * Returns: the last state.
+ */
+MilterClientContextState milter_client_context_get_last_state(MilterClientContext *context);
+
+/**
+ * milter_client_context_set_status:
+ * @context: a %MilterClientContext.
+ * @status: a %MilterStatus.
+ *
+ * Sets the current status.
+ */
+void                 milter_client_context_set_status        (MilterClientContext *context,
+                                                              MilterStatus status);
+
+/**
+ * milter_client_context_get_status:
+ * @context: a %MilterClientContext.
+ *
+ * Gets the current status.
+ *
+ * Returns: the current status.
+ */
+MilterStatus             milter_client_context_get_status    (MilterClientContext *context);
+
 /**
  * milter_client_context_set_option:
  * @context: a %MilterClientContext.
