@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -33,6 +33,21 @@
 #include <milter/core/milter-marshalers.h>
 #include "../client.h"
 
+static guint milter_client_log_handler_id = 0;
+
+void
+milter_client_init (void)
+{
+    milter_client_log_handler_id = MILTER_GLIB_LOG_DELEGATE("milter-client");
+}
+
+void
+milter_client_quit (void)
+{
+    g_log_remove_handler("milter-client", milter_client_log_handler_id);
+}
+
+
 enum
 {
     CONNECTION_ESTABLISHED,
@@ -66,6 +81,7 @@ struct _MilterClientPrivate
     gchar *default_unix_socket_group;
     gboolean default_remove_unix_socket_on_close;
     gboolean remove_unix_socket_on_create;
+    guint suspend_time_on_unacceptable;
 };
 
 typedef struct _MilterClientProcessData
@@ -75,7 +91,9 @@ typedef struct _MilterClientProcessData
     MilterClientContext *context;
 } MilterClientProcessData;
 
-MILTER_DEFINE_ERROR_EMITTABLE_TYPE(MilterClient, milter_client, G_TYPE_OBJECT)
+#define _milter_client_get_type milter_client_get_type
+MILTER_DEFINE_ERROR_EMITTABLE_TYPE(MilterClient, _milter_client, G_TYPE_OBJECT)
+#undef _milter_client_get_type
 
 static void dispose        (GObject         *object);
 static void set_property   (GObject         *object,
@@ -97,7 +115,7 @@ static void   cb_finished  (MilterClientContext *context,
                             gpointer _data);
 
 static void
-milter_client_class_init (MilterClientClass *klass)
+_milter_client_class_init (MilterClientClass *klass)
 {
     GObjectClass *gobject_class;
     MilterClientClass *client_class = NULL;
@@ -134,7 +152,7 @@ milter_client_class_init (MilterClientClass *klass)
 }
 
 static void
-milter_client_init (MilterClient *client)
+_milter_client_init (MilterClient *client)
 {
     MilterClientPrivate *priv;
     GMainContext *main_context;
@@ -162,6 +180,8 @@ milter_client_init (MilterClient *client)
     priv->default_unix_socket_group = NULL;
     priv->default_remove_unix_socket_on_close = TRUE;
     priv->remove_unix_socket_on_create = TRUE;
+    priv->suspend_time_on_unacceptable =
+        MILTER_CLIENT_DEFAULT_SUSPEND_TIME_ON_UNACCEPTABLE;
 }
 
 static void
@@ -227,7 +247,7 @@ dispose (GObject *object)
         priv->default_unix_socket_group = NULL;
     }
 
-    G_OBJECT_CLASS(milter_client_parent_class)->dispose(object);
+    G_OBJECT_CLASS(_milter_client_parent_class)->dispose(object);
 }
 
 static void
@@ -290,20 +310,24 @@ change_unix_socket_group (MilterClient *client, struct sockaddr_un *address_un)
     if (!group) {
         if (errno == 0) {
             milter_error(
-                "failed to find group entry for UNIX socket group: %s",
-                socket_group);
+                "[client][error][unix] "
+                "failed to find group entry for UNIX socket group: <%s>: <%s>",
+                address_un->sun_path, socket_group);
         } else {
             milter_error(
-                "failed to get group entry for UNIX socket group: %s: %s",
-                socket_group, g_strerror(errno));
+                "[client][error][unix] "
+                "failed to get group entry for UNIX socket group: "
+                "<%s>: <%s>: %s",
+                address_un->sun_path, socket_group, g_strerror(errno));
         }
         return;
     }
 
     if (chown(address_un->sun_path, -1, group->gr_gid) == -1) {
         milter_error(
-            "failed to change UNIX socket group: %s: %s",
-            socket_group, g_strerror(errno));
+            "[client][error][unix] "
+            "failed to change UNIX socket group: <%s>: <%s>: %s",
+            address_un->sun_path, socket_group, g_strerror(errno));
     }
 }
 
@@ -409,8 +433,10 @@ cb_idle_free_data (gpointer _data)
     GList *processing_data, *process_data;
     guint n_processing_data;
     GString *rest_process;
+    guint tag;
 
-    milter_debug("[client][finish] %p", data);
+    tag = milter_agent_get_tag(MILTER_AGENT(data->context));
+    milter_debug("[%u] [client][finish]", tag);
     data->priv->processing_data =
         g_list_remove(data->priv->processing_data, data);
     data->priv->n_processing_data--;
@@ -420,7 +446,7 @@ cb_idle_free_data (gpointer _data)
     n_processing_data = data->priv->n_processing_data;
     g_mutex_lock(data->priv->quit_mutex);
     if (data->priv->quitting && n_processing_data == 0) {
-        milter_debug("[client][loop][quit]");
+        milter_debug("[%u] [client][loop][quit]", tag);
         g_main_loop_quit(data->priv->main_loop);
     }
     g_mutex_unlock(data->priv->quit_mutex);
@@ -431,13 +457,16 @@ cb_idle_free_data (gpointer _data)
     for (process_data = processing_data;
          process_data;
          process_data = g_list_next(process_data)) {
-        g_string_append_printf(rest_process, "<%p>, ", process_data->data);
+        MilterClientProcessData *_process_data = process_data->data;
+        g_string_append_printf(
+            rest_process, "<%u>, ",
+            milter_agent_get_tag(MILTER_AGENT(_process_data->context)));
     }
     if (processing_data)
         g_string_truncate(rest_process, rest_process->len - 2);
     g_string_append(rest_process, "]");
     g_list_free(processing_data);
-    milter_debug("[client][rest] %s", rest_process->str);
+    milter_debug("[%u] [client][rest] %s", tag, rest_process->str);
     g_string_free(rest_process, TRUE);
 
     return FALSE;
@@ -487,7 +516,8 @@ cb_timeout_client_channel_setup (gpointer user_data)
     data->client = client;
     data->context = context;
 
-    milter_debug("[client][start] %p", data);
+    milter_debug("[%u] [client][start]",
+                 milter_agent_get_tag(MILTER_AGENT(data->context)));
 
     g_signal_connect(context, "finished", G_CALLBACK(cb_finished), data);
 
@@ -519,12 +549,15 @@ process_client_channel (MilterClient *client, GIOChannel *channel)
 static gboolean
 accept_client (gint server_fd, MilterClient *client)
 {
+    MilterClientPrivate *priv;
     gint client_fd;
     GIOChannel *client_channel;
     MilterGenericSocketAddress address;
     gchar *spec;
     socklen_t address_size;
 
+    priv = MILTER_CLIENT_GET_PRIVATE(client);
+
     address_size = sizeof(address);
     memset(&address, '\0', address_size);
     client_fd = accept(server_fd, (struct sockaddr *)(&address), &address_size);
@@ -538,6 +571,21 @@ accept_client (gint server_fd, MilterClient *client)
         milter_error_emittable_emit(MILTER_ERROR_EMITTABLE(client),
                                     error);
         g_error_free(error);
+
+        if (errno == EMFILE) {
+            guint suspend_time;
+
+            suspend_time =
+                milter_client_get_suspend_time_on_unacceptable(client);
+            milter_warning("[client][accept][suspend] "
+                           "too many file is opened. "
+                           "suspend accepting connection in %d seconds",
+                           suspend_time);
+            g_usleep(suspend_time * G_USEC_PER_SEC);
+            milter_warning("[client][accept][resume] "
+                           "resume accepting connection.");
+        }
+
         return TRUE;
     }
 
@@ -868,6 +916,31 @@ milter_client_set_default_remove_unix_socket_on_close (MilterClient *client,
     MILTER_CLIENT_GET_PRIVATE(client)->default_remove_unix_socket_on_close = remove;
 }
 
+guint
+milter_client_get_suspend_time_on_unacceptable (MilterClient *client)
+{
+    MilterClientClass *klass;
+
+    klass = MILTER_CLIENT_GET_CLASS(client);
+    if (klass->get_suspend_time_on_unacceptable)
+        return klass->get_suspend_time_on_unacceptable(client);
+    else
+        return MILTER_CLIENT_GET_PRIVATE(client)->suspend_time_on_unacceptable;
+}
+
+void
+milter_client_set_suspend_time_on_unacceptable (MilterClient *client,
+                                                guint suspend_time)
+{
+    MilterClientClass *klass;
+
+    klass = MILTER_CLIENT_GET_CLASS(client);
+    if (klass->set_suspend_time_on_unacceptable)
+        klass->set_suspend_time_on_unacceptable(client, suspend_time);
+    else
+        MILTER_CLIENT_GET_PRIVATE(client)->suspend_time_on_unacceptable = suspend_time;
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
