@@ -118,6 +118,10 @@ milter_status_get_type (void)
       { MILTER_STATUS_SKIP, "MILTER_STATUS_SKIP", "skip" },
       { MILTER_STATUS_ALL_OPTIONS, "MILTER_STATUS_ALL_OPTIONS", "all-options" },
       { MILTER_STATUS_PROGRESS, "MILTER_STATUS_PROGRESS", "progress" },
+      { MILTER_STATUS_ABORT, "MILTER_STATUS_ABORT", "abort" },
+      { MILTER_STATUS_QUARANTINE, "MILTER_STATUS_QUARANTINE", "quarantine" },
+      { MILTER_STATUS_STOP, "MILTER_STATUS_STOP", "stop" },
+      { MILTER_STATUS_ERROR, "MILTER_STATUS_ERROR", "error" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static ("MilterStatus", values);
