@@ -1365,7 +1365,52 @@ _milter_marshal_ENUM__VOID (GClosure     *closure,
   g_value_set_enum (return_value, v_return);
 }
 
-/* ENUM:STRING,UINT (./milter-marshalers.list:31) */
+/* ENUM:ENUM (./milter-marshalers.list:31) */
+extern void _milter_marshal_ENUM__ENUM (GClosure     *closure,
+                                        GValue       *return_value,
+                                        guint         n_param_values,
+                                        const GValue *param_values,
+                                        gpointer      invocation_hint,
+                                        gpointer      marshal_data);
+void
+_milter_marshal_ENUM__ENUM (GClosure     *closure,
+                            GValue       *return_value G_GNUC_UNUSED,
+                            guint         n_param_values,
+                            const GValue *param_values,
+                            gpointer      invocation_hint G_GNUC_UNUSED,
+                            gpointer      marshal_data)
+{
+  typedef gint (*GMarshalFunc_ENUM__ENUM) (gpointer     data1,
+                                           gint         arg_1,
+                                           gpointer     data2);
+  register GMarshalFunc_ENUM__ENUM callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+  gint v_return;
+
+  g_return_if_fail (return_value != NULL);
+  g_return_if_fail (n_param_values == 2);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_ENUM__ENUM) (marshal_data ? marshal_data : cc->callback);
+
+  v_return = callback (data1,
+                       g_marshal_value_peek_enum (param_values + 1),
+                       data2);
+
+  g_value_set_enum (return_value, v_return);
+}
+
+/* ENUM:STRING,UINT (./milter-marshalers.list:32) */
 extern void _milter_marshal_ENUM__STRING_UINT (GClosure     *closure,
                                                GValue       *return_value,
                                                guint         n_param_values,
@@ -1412,7 +1457,7 @@ _milter_marshal_ENUM__STRING_UINT (GClosure     *closure,
   g_value_set_enum (return_value, v_return);
 }
 
-/* ENUM:STRING,UINT64 (./milter-marshalers.list:32) */
+/* ENUM:STRING,UINT64 (./milter-marshalers.list:33) */
 extern void _milter_marshal_ENUM__STRING_UINT64 (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
