@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -62,6 +62,7 @@ struct _MilterManagerConfigurationClass
     gboolean     (*load_custom)   (MilterManagerConfiguration *configuration,
                                    const gchar                *file_name,
                                    GError                    **error);
+    void         (*maintain)      (MilterManagerConfiguration *configuration);
     void         (*to_xml)        (MilterManagerConfiguration *configuration,
                                    GString                    *xml,
                                    guint                       indent);
@@ -210,6 +211,24 @@ void          milter_manager_configuration_set_package_options
                                      (MilterManagerConfiguration *configuration,
                                       const gchar                *options);
 
+guint         milter_manager_configuration_get_maintenance_interval
+                                     (MilterManagerConfiguration *configuration);
+void          milter_manager_configuration_set_maintenance_interval
+                                     (MilterManagerConfiguration *configuration,
+                                      guint                       n_sessions);
+
+guint         milter_manager_configuration_get_suspend_time_on_unacceptable
+                                     (MilterManagerConfiguration *configuration);
+void          milter_manager_configuration_set_suspend_time_on_unacceptable
+                                     (MilterManagerConfiguration *configuration,
+                                      guint                       suspend_time);
+
+const gchar  *milter_manager_configuration_get_custom_configuration_directory
+                                     (MilterManagerConfiguration *configuration);
+void          milter_manager_configuration_set_custom_configuration_directory
+                                     (MilterManagerConfiguration *configuration,
+                                      const gchar                *directory);
+
 void          milter_manager_configuration_add_egg
                                      (MilterManagerConfiguration *configuration,
                                       MilterManagerEgg *egg);
@@ -250,6 +269,10 @@ void          milter_manager_configuration_clear_applicable_conditions
 void          milter_manager_configuration_clear
                                      (MilterManagerConfiguration *configuration);
 
+void          milter_manager_configuration_session_finished
+                                     (MilterManagerConfiguration *configuration);
+
+
 gchar        *milter_manager_configuration_to_xml
                                      (MilterManagerConfiguration *configuration);
 void          milter_manager_configuration_to_xml_string
