@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -268,8 +268,11 @@ static MilterManagerLeaderState
 next_state (MilterManagerLeader *leader,
             MilterManagerLeaderState state)
 {
+    MilterManagerLeaderPrivate *priv;
     MilterManagerLeaderState next_state;
 
+    priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
+
     switch (state) {
       case MILTER_MANAGER_LEADER_STATE_NEGOTIATE:
         next_state = MILTER_MANAGER_LEADER_STATE_NEGOTIATE_REPLIED;
@@ -300,15 +303,11 @@ next_state (MilterManagerLeader *leader,
         break;
       case MILTER_MANAGER_LEADER_STATE_BODY:
       case MILTER_MANAGER_LEADER_STATE_BODY_REPLIED:
-      {
-        MilterManagerLeaderPrivate *priv;
-        priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
         if (!priv->sent_end_of_message)
             next_state = MILTER_MANAGER_LEADER_STATE_BODY_REPLIED;
         else
             next_state = MILTER_MANAGER_LEADER_STATE_END_OF_MESSAGE;
         break;
-      }
       case MILTER_MANAGER_LEADER_STATE_END_OF_MESSAGE:
         next_state = MILTER_MANAGER_LEADER_STATE_END_OF_MESSAGE_REPLIED;
         break;
@@ -321,11 +320,14 @@ next_state (MilterManagerLeader *leader,
       default:
         {
             gchar *inspected_state;
+            guint tag;
 
+            tag = milter_agent_get_tag(MILTER_AGENT(priv->client_context));
             inspected_state =
                 milter_utils_inspect_enum(MILTER_TYPE_MANAGER_LEADER_STATE,
                                           state);
-            g_print("ERROR! - invalid state: %s|FIXME\n", inspected_state);
+            milter_error("[%u] [leader][error][invalid-state] %s",
+                         tag, inspected_state);
             g_free(inspected_state);
         }
         next_state = MILTER_MANAGER_LEADER_STATE_INVALID;
@@ -367,19 +369,23 @@ reply (MilterManagerLeader *leader, MilterStatus status)
             g_signal_emit_by_name(priv->client_context, signal_name, status);
         } else {
             GError *error;
-            gchar *state_name;
+            guint tag;
+            gchar *state_name, *status_name;
 
+            tag = milter_agent_get_tag(MILTER_AGENT(priv->client_context));
             state_name =
                 milter_utils_get_enum_nick_name(MILTER_TYPE_MANAGER_LEADER_STATE,
                                                 priv->state);
+            status_name = milter_utils_get_enum_nick_name(MILTER_TYPE_STATUS,
+                                                          status);
             error = g_error_new(MILTER_MANAGER_LEADER_ERROR,
                                 MILTER_MANAGER_LEADER_ERROR_INVALID_STATE,
-                                "invalid state to reply: %s",
-                                state_name);
+                                "invalid state to reply: %s: %s",
+                                state_name, status_name);
             g_free(state_name);
-            milter_error("[leader][error] %s", error->message);
-            milter_error_emittable_emit(MILTER_ERROR_EMITTABLE(priv->children),
-                                        error);
+            g_free(status_name);
+            milter_error("[%u] [leader][error] %s", tag, error->message);
+            milter_error_emittable_emit(MILTER_ERROR_EMITTABLE(leader), error);
             g_error_free(error);
             return;
         }
@@ -588,13 +594,23 @@ cb_quarantine (MilterReplySignals *_reply,
 static void
 cb_connection_failure (MilterReplySignals *_reply, gpointer user_data)
 {
-    milter_debug("[leader][unsupported][connection-failure]");
+    MilterManagerLeader *leader = user_data;
+    MilterManagerLeaderPrivate *priv;
+
+    priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
+    milter_debug("[%u] [leader][unsupported][connection-failure]",
+                 milter_agent_get_tag(MILTER_AGENT(priv->client_context)));
 }
 
 static void
 cb_shutdown (MilterReplySignals *_reply, gpointer user_data)
 {
-    milter_debug("[leader][unsupported][shutdown]");
+    MilterManagerLeader *leader = user_data;
+    MilterManagerLeaderPrivate *priv;
+
+    priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
+    milter_debug("[%u] [leader][unsupported][shutdown]",
+                 milter_agent_get_tag(MILTER_AGENT(priv->client_context)));
 }
 
 static void
@@ -616,7 +632,10 @@ cb_abort (MilterReplySignals *_reply, gpointer user_data)
     state_name =
         milter_utils_get_enum_nick_name(MILTER_TYPE_MANAGER_LEADER_STATE,
                                         priv->state);
-    milter_statistics("[abort][%s]", state_name);
+    milter_debug("[%u] [leader][abort][%s]",
+                 milter_agent_get_tag(MILTER_AGENT(priv->client_context)),
+                 state_name);
+    g_free(state_name);
     milter_agent_shutdown(MILTER_AGENT(priv->client_context));
     milter_finished_emittable_emit(MILTER_FINISHED_EMITTABLE(leader));
 }
@@ -625,19 +644,28 @@ static void
 cb_error (MilterErrorEmittable *emittable, GError *error, gpointer user_data)
 {
     MilterManagerLeader *leader = user_data;
+    MilterManagerLeaderPrivate *priv;
 
-    milter_error("[leader][error] %s", error->message);
+    priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
+    milter_error("[%u] [leader][error] %s",
+                 milter_agent_get_tag(MILTER_AGENT(priv->client_context)),
+                 error->message);
     milter_error_emittable_emit(MILTER_ERROR_EMITTABLE(leader),
                                 error);
 
-    /* If all milters do not response on negotiation, send "reject" */
     if (error->domain != MILTER_MANAGER_CHILDREN_ERROR)
         return;
 
-    if (error->code == MILTER_MANAGER_CHILDREN_ERROR_NO_NEGOTIATION_RESPONSE)
+    /* If all milters do not response on negotiation, send "reject" */
+    if (error->code == MILTER_MANAGER_CHILDREN_ERROR_NO_NEGOTIATION_RESPONSE) {
         reply(leader, MILTER_STATUS_REJECT);
-    else
-        reply(leader, MILTER_STATUS_TEMPORARY_FAILURE);
+    } else {
+        MilterStatus fallback_status;
+
+        fallback_status =
+            milter_manager_configuration_get_fallback_status(priv->configuration);
+        reply(leader, fallback_status);
+    }
 }
 
 static void
@@ -723,6 +751,7 @@ milter_manager_leader_negotiate (MilterManagerLeader *leader,
                                  MilterMacrosRequests *macros_requests)
 {
     MilterManagerLeaderPrivate *priv;
+    guint tag;
 
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
     priv->state = MILTER_MANAGER_LEADER_STATE_NEGOTIATE;
@@ -733,10 +762,13 @@ milter_manager_leader_negotiate (MilterManagerLeader *leader,
     if (!priv->children)
         return MILTER_STATUS_NOT_CHANGE;
 
+    tag = milter_agent_get_tag(MILTER_AGENT(priv->client_context));
+    milter_manager_children_set_tag(priv->children, tag);
     setup_children_signals(leader, priv->children);
     milter_manager_children_set_launcher_channel(priv->children,
                                                  priv->launcher_read_channel,
                                                  priv->launcher_write_channel);
+    milter_debug("[%u] [leader][setup][children]", tag);
 
     if (milter_manager_children_negotiate(priv->children, option,
                                           macros_requests))
@@ -962,13 +994,9 @@ milter_manager_leader_quit (MilterManagerLeader *leader)
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
     priv->state = MILTER_MANAGER_LEADER_STATE_QUIT;
 
-    if (!priv->children)
-        return MILTER_STATUS_NOT_CHANGE;
-
-    if (milter_manager_children_quit(priv->children))
-        return MILTER_STATUS_PROGRESS;
-    else
-        return MILTER_STATUS_NOT_CHANGE; /* FIXME: reject or accept */
+    if (priv->children)
+        milter_manager_children_quit(priv->children);
+    return MILTER_STATUS_DEFAULT;
 }
 
 MilterStatus
@@ -979,13 +1007,9 @@ milter_manager_leader_abort (MilterManagerLeader *leader)
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
     priv->state = MILTER_MANAGER_LEADER_STATE_ABORT;
 
-    if (!priv->children)
-        return MILTER_STATUS_NOT_CHANGE;
-
-    if (milter_manager_children_abort(priv->children))
-        return MILTER_STATUS_PROGRESS;
-    else
-        return MILTER_STATUS_NOT_CHANGE; /* FIXME: reject or accept */
+    if (priv->children)
+        milter_manager_children_abort(priv->children);
+    return MILTER_STATUS_DEFAULT;
 }
 
 void
@@ -1010,6 +1034,12 @@ milter_manager_leader_timeout (MilterManagerLeader *leader)
     milter_manager_leader_quit(leader);
 }
 
+MilterManagerConfiguration *
+milter_manager_leader_get_configuration (MilterManagerLeader *leader)
+{
+    return MILTER_MANAGER_LEADER_GET_PRIVATE(leader)->configuration;
+}
+
 void
 milter_manager_leader_set_launcher_channel (MilterManagerLeader *leader,
                                             GIOChannel *read_channel,
