@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -53,9 +53,19 @@ static gboolean option_verbose = FALSE;
 
 static gboolean io_detached = FALSE;
 
-static void (*default_sigint_handler) (int signum);
-static void (*default_sigterm_handler) (int signum);
-static void (*default_sighup_handler) (int signum);
+static guint milter_manager_log_handler_id = 0;
+
+static struct sigaction default_sigsegv_action;
+static struct sigaction default_sigabort_action;
+static struct sigaction default_sigint_action;
+static struct sigaction default_sigterm_action;
+static struct sigaction default_sighup_action;
+
+static gboolean set_sigsegv_action = TRUE;
+static gboolean set_sigabort_action = TRUE;
+static gboolean set_sigint_action = TRUE;
+static gboolean set_sigterm_action = TRUE;
+static gboolean set_sighup_action = TRUE;
 
 #define milter_manager_error(...) G_STMT_START  \
 {                                               \
@@ -155,6 +165,9 @@ milter_manager_init (int *argc, char ***argv)
     textdomain(GETTEXT_PACKAGE);
 
     milter_init();
+    milter_client_init();
+    milter_server_init();
+    milter_manager_log_handler_id = MILTER_GLIB_LOG_DELEGATE("milter-manager");
 
     option_context = g_option_context_new(NULL);
     g_option_context_add_main_entries(option_context, option_entries, NULL);
@@ -199,23 +212,35 @@ milter_manager_quit (void)
         g_free(option_spec);
         option_spec = NULL;
     }
+
+    g_log_remove_handler("milter-manager", milter_manager_log_handler_id);
+    milter_server_quit();
+    milter_client_quit();
+    milter_quit();
 }
 
 static void
 shutdown_client (int signum)
 {
+    struct sigaction default_action;
+
     if (the_manager)
         milter_client_shutdown(MILTER_CLIENT(the_manager));
 
     switch (signum) {
     case SIGINT:
-        signal(SIGINT, default_sigint_handler);
+        if (sigaction(SIGINT, &default_sigint_action, NULL) != -1)
+            set_sigint_action = FALSE;
         break;
     case SIGTERM:
-        signal(SIGTERM, default_sigterm_handler);
+        if (sigaction(SIGTERM, &default_sigterm_action, NULL) != -1)
+            set_sigterm_action = FALSE;
         break;
     default:
-        signal(signum, SIG_DFL);
+        default_action.sa_handler = SIG_DFL;
+        sigemptyset(&default_action.sa_mask);
+        default_action.sa_flags = 0;
+        sigaction(signum, &default_action, NULL);
         break;
     }
 }
@@ -597,11 +622,11 @@ apply_command_line_options (MilterManagerConfiguration *config)
 }
 
 static void
-append_user_local_configuration_path (MilterManagerConfiguration *config)
+append_custom_configuration_directory (MilterManagerConfiguration *config)
 {
     uid_t uid;
     struct passwd *password;
-    gchar *user_local_config_path;
+    gchar *custom_config_directory;
 
     uid = geteuid();
     if (uid == 0)
@@ -609,25 +634,88 @@ append_user_local_configuration_path (MilterManagerConfiguration *config)
 
     password = getpwuid(uid);
 
-    user_local_config_path = g_build_filename(password->pw_dir,
-                                              ".milter-manager",
-                                              NULL);
-    if (!g_file_test(user_local_config_path, G_FILE_TEST_EXISTS)) {
-        if (g_mkdir(user_local_config_path, 0700) == -1) {
-            milter_manager_error("failed to create user local "
+    custom_config_directory =
+        g_strdup(milter_manager_configuration_get_custom_configuration_directory(config));
+    if (!custom_config_directory)
+        custom_config_directory = g_build_filename(password->pw_dir,
+                                                   ".milter-manager",
+                                                   NULL);
+    if (!g_file_test(custom_config_directory, G_FILE_TEST_EXISTS)) {
+        if (g_mkdir(custom_config_directory, 0700) == -1) {
+            milter_manager_error("failed to create custom "
                                  "configuration directory: %s: %s",
-                                 user_local_config_path,
+                                 custom_config_directory,
                                  g_strerror(errno));
             return;
         }
     }
 
     milter_manager_configuration_append_load_path(config,
-                                                  user_local_config_path);
+                                                  custom_config_directory);
+    g_free(custom_config_directory);
+
     milter_manager_configuration_reload(config);
     apply_command_line_options(config);
 }
 
+static gchar report_stack_trace_window[4096];
+static void
+report_stack_trace (int signum)
+{
+    int fds[2];
+    int in_fd;
+    const char *label = "unknown";
+    int original_stdout_fileno;
+    gssize i, last_new_line, size;
+
+    switch (signum) {
+    case SIGSEGV:
+        label = "SEGV";
+        if (sigaction(SIGSEGV, &default_sigsegv_action, NULL) != -1)
+            set_sigsegv_action = FALSE;
+        break;
+    case SIGABRT:
+        label = "ABORT";
+        if (sigaction(SIGABRT, &default_sigabort_action, NULL) != -1)
+            set_sigabort_action = FALSE;
+        break;
+    }
+
+    if (pipe(fds) == -1) {
+        milter_critical("[%s] unable to open pipe for collecting stack trace",
+                        label);
+        return;
+    }
+
+    original_stdout_fileno = dup(STDOUT_FILENO);
+    dup2(fds[1], STDOUT_FILENO);
+    g_on_error_stack_trace(g_get_prgname());
+    dup2(original_stdout_fileno, STDOUT_FILENO);
+    close(original_stdout_fileno);
+    close(fds[1]);
+
+    in_fd = fds[0];
+    while ((size = read(in_fd,
+                        report_stack_trace_window,
+                        sizeof(report_stack_trace_window))) > 0) {
+        last_new_line = 0;
+        for (i = 0; i < size; i++) {
+            if (report_stack_trace_window[i] == '\n') {
+                report_stack_trace_window[i] = '\0';
+                milter_critical("%s", report_stack_trace_window + last_new_line);
+                last_new_line = i + 1;
+            }
+        }
+        if (last_new_line < size) {
+            report_stack_trace_window[size] = '\0';
+            milter_critical("%s", report_stack_trace_window + last_new_line);
+        }
+    }
+
+    close(fds[0]);
+
+    shutdown_client(SIGINT);
+}
 
 gboolean
 milter_manager_main (void)
@@ -638,6 +726,9 @@ milter_manager_main (void)
     MilterManagerConfiguration *config;
     gboolean daemon;
     gchar *pid_file = NULL;
+    struct sigaction report_stack_trace_action;
+    struct sigaction shutdown_client_action;
+    struct sigaction reload_configuration_action;
 
     config = milter_manager_configuration_new(NULL);
     if (option_config_dir)
@@ -671,7 +762,7 @@ milter_manager_main (void)
         return FALSE;
     }
 
-    append_user_local_configuration_path(config);
+    append_custom_configuration_directory(config);
 
     if (option_show_config) {
         gchar *dumped_config;
@@ -713,14 +804,43 @@ milter_manager_main (void)
     }
 
     the_manager = manager;
-    default_sigint_handler = signal(SIGINT, shutdown_client);
-    default_sigterm_handler = signal(SIGTERM, shutdown_client);
-    default_sighup_handler = signal(SIGHUP, reload_configuration);
+
+#define SETUP_SIGNAL_ACTION(handler)            \
+    handler ## _action.sa_handler = handler;    \
+    sigemptyset(&handler ## _action.sa_mask);   \
+    handler ## _action.sa_flags = 0
+
+    SETUP_SIGNAL_ACTION(report_stack_trace);
+    SETUP_SIGNAL_ACTION(shutdown_client);
+    SETUP_SIGNAL_ACTION(reload_configuration);
+#undef SETUP_SIGNAL_ACTION
+
+#define SET_SIGNAL_ACTION(SIGNAL, signal, action)               \
+    if (sigaction(SIG ## SIGNAL,                                \
+                  &action,                                      \
+                  &default_sig ## signal ## _action) == -1)     \
+        set_sig ## signal ## _action = FALSE
+
+    SET_SIGNAL_ACTION(SEGV, segv, report_stack_trace_action);
+    SET_SIGNAL_ACTION(ABRT, abort, report_stack_trace_action);
+    SET_SIGNAL_ACTION(INT, int, shutdown_client_action);
+    SET_SIGNAL_ACTION(TERM, term, shutdown_client_action);
+    SET_SIGNAL_ACTION(HUP, hup, reload_configuration_action);
+#undef SET_SIGNAL_ACTION
+
     if (!milter_client_main(client))
         milter_manager_error("failed to start milter-manager process.");
-    signal(SIGHUP, default_sighup_handler);
-    signal(SIGTERM, default_sigterm_handler);
-    signal(SIGINT, default_sigint_handler);
+
+#define UNSET_SIGNAL_ACTION(SIGNAL, signal)                             \
+    if (set_sig ## signal ## _action)                                   \
+        sigaction(SIG ## SIGNAL, &default_sig ## signal ## _action, NULL)
+
+    UNSET_SIGNAL_ACTION(SEGV, segv);
+    UNSET_SIGNAL_ACTION(ABRT, abort);
+    UNSET_SIGNAL_ACTION(INT, int);
+    UNSET_SIGNAL_ACTION(TERM, term);
+    UNSET_SIGNAL_ACTION(HUP, hup);
+#undef UNSET_SIGNAL_ACTION
 
     if (controller)
         g_object_unref(controller);
