@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -71,6 +71,11 @@ static gchar *get_default_connection_spec (MilterClient *client);
 static guint  get_unix_socket_mode        (MilterClient *client);
 static const gchar *get_unix_socket_group (MilterClient *client);
 static gboolean is_remove_unix_socket_on_close (MilterClient *client);
+static guint  get_suspend_time_on_unacceptable
+                                          (MilterClient *client);
+static void   set_suspend_time_on_unacceptable
+                                          (MilterClient *client,
+                                           guint         suspend_time);
 static void   cb_leader_finished          (MilterFinishedEmittable *emittable,
                                            gpointer user_data);
 
@@ -93,6 +98,8 @@ milter_manager_class_init (MilterManagerClass *klass)
     client_class->get_unix_socket_mode        = get_unix_socket_mode;
     client_class->get_unix_socket_group       = get_unix_socket_group;
     client_class->is_remove_unix_socket_on_close = is_remove_unix_socket_on_close;
+    client_class->get_suspend_time_on_unacceptable = get_suspend_time_on_unacceptable;
+    client_class->set_suspend_time_on_unacceptable = set_suspend_time_on_unacceptable;
 
     spec = g_param_spec_object("configuration",
                                "Configuration",
@@ -206,7 +213,8 @@ cb_client_negotiate (MilterClientContext *context, MilterOption *option,
 
     inspected_option = milter_option_inspect(option);
     inspected_macros_requests = milter_utils_inspect_object(G_OBJECT(macros_requests));
-    milter_debug("[manager][receive][negotiate] <%s>:<%s>",
+    milter_debug("[%u] [manager][receive][negotiate] <%s>:<%s>",
+                 milter_agent_get_tag(MILTER_AGENT(context)),
                  inspected_option, inspected_macros_requests);
     g_free(inspected_option);
     g_free(inspected_macros_requests);
@@ -223,7 +231,9 @@ cb_client_connect (MilterClientContext *context, const gchar *host_name,
     gchar *spec;
 
     spec = milter_connection_address_to_spec(address);
-    milter_debug("[manager][receive][connect] <%s>:<%s>", host_name, spec);
+    milter_debug("[%u] [manager][receive][connect] <%s>:<%s>",
+                 milter_agent_get_tag(MILTER_AGENT(context)),
+                 host_name, spec);
     g_free(spec);
 
     return milter_manager_leader_connect(leader, host_name,
@@ -236,7 +246,9 @@ cb_client_helo (MilterClientContext *context, const gchar *fqdn,
 {
     MilterManagerLeader *leader = user_data;
 
-    milter_debug("[manager][receive][helo] <%s>", fqdn);
+    milter_debug("[%u] [manager][receive][helo] <%s>",
+                 milter_agent_get_tag(MILTER_AGENT(context)),
+                 fqdn);
 
     return milter_manager_leader_helo(leader, fqdn);
 }
@@ -247,7 +259,9 @@ cb_client_envelope_from (MilterClientContext *context, const gchar *from,
 {
     MilterManagerLeader *leader = user_data;
 
-    milter_debug("[manager][receive][envelope-from] <%s>", from);
+    milter_debug("[%u] [manager][receive][envelope-from] <%s>",
+                 milter_agent_get_tag(MILTER_AGENT(context)),
+                 from);
 
     return milter_manager_leader_envelope_from(leader, from);
 }
@@ -258,7 +272,9 @@ cb_client_envelope_recipient (MilterClientContext *context,
 {
     MilterManagerLeader *leader = user_data;
 
-    milter_debug("[manager][receive][envelope-recipient] <%s>", recipient);
+    milter_debug("[%u] [manager][receive][envelope-recipient] <%s>",
+                 milter_agent_get_tag(MILTER_AGENT(context)),
+                 recipient);
 
     return milter_manager_leader_envelope_recipient(leader, recipient);
 }
@@ -268,7 +284,8 @@ cb_client_data (MilterClientContext *context, gpointer user_data)
 {
     MilterManagerLeader *leader = user_data;
 
-    milter_debug("[manager][receive][data]");
+    milter_debug("[%u] [manager][receive][data]",
+                 milter_agent_get_tag(MILTER_AGENT(context)));
 
     return milter_manager_leader_data(leader);
 }
@@ -279,7 +296,9 @@ cb_client_unknown (MilterClientContext *context, const gchar *command,
 {
     MilterManagerLeader *leader = user_data;
 
-    milter_debug("[manager][receive][unknown] <%s>", command);
+    milter_debug("[%u] [manager][receive][unknown] <%s>",
+                 milter_agent_get_tag(MILTER_AGENT(context)),
+                 command);
 
     return milter_manager_leader_unknown(leader, command);
 }
@@ -291,7 +310,9 @@ cb_client_header (MilterClientContext *context,
 {
     MilterManagerLeader *leader = user_data;
 
-    milter_debug("[manager][receive][header] <%s>=<%s>", name, value);
+    milter_debug("[%u] [manager][receive][header] <%s>=<%s>",
+                 milter_agent_get_tag(MILTER_AGENT(context)),
+                 name, value);
 
     return milter_manager_leader_header(leader, name, value);
 }
@@ -301,7 +322,8 @@ cb_client_end_of_header (MilterClientContext *context, gpointer user_data)
 {
     MilterManagerLeader *leader = user_data;
 
-    milter_debug("[manager][receive][end-of-header]");
+    milter_debug("[%u] [manager][receive][end-of-header]",
+                 milter_agent_get_tag(MILTER_AGENT(context)));
 
     return milter_manager_leader_end_of_header(leader);
 }
@@ -312,27 +334,42 @@ cb_client_body (MilterClientContext *context, const gchar *chunk, gsize size,
 {
     MilterManagerLeader *leader = user_data;
 
-    milter_debug("[manager][receive][body] <%" G_GSIZE_FORMAT ">", size);
+    milter_debug("[%u] [manager][receive][body] <%" G_GSIZE_FORMAT ">",
+                 milter_agent_get_tag(MILTER_AGENT(context)),
+                 size);
 
     return milter_manager_leader_body(leader, chunk, size);
 }
 
 static MilterStatus
-cb_client_end_of_message (MilterClientContext *context, gpointer user_data)
+cb_client_end_of_message (MilterClientContext *context,
+                          const gchar *chunk, gsize size,
+                          gpointer user_data)
 {
     MilterManagerLeader *leader = user_data;
 
-    milter_debug("[manager][receive][end-of-message]");
+    milter_debug("[%u] [manager][receive][end-of-message] "
+                 "<%" G_GSIZE_FORMAT ">",
+                 milter_agent_get_tag(MILTER_AGENT(context)),
+                 size);
 
-    return milter_manager_leader_end_of_message(leader, NULL, 0);
+    return milter_manager_leader_end_of_message(leader, chunk, size);
 }
 
 static MilterStatus
-cb_client_abort (MilterClientContext *context, gpointer user_data)
+cb_client_abort (MilterClientContext *context, MilterClientContextState state,
+                 gpointer user_data)
 {
     MilterManagerLeader *leader = user_data;
+    gchar *state_name;
 
-    milter_debug("[manager][receive][abort]");
+    state_name =
+        milter_utils_get_enum_nick_name(MILTER_TYPE_CLIENT_CONTEXT_STATE,
+                                        state);
+    milter_debug("[%u] [manager][receive][abort][%s]",
+                 milter_agent_get_tag(MILTER_AGENT(context)),
+                 state_name);
+    g_free(state_name);
 
     return milter_manager_leader_abort(leader);
 }
@@ -345,7 +382,8 @@ cb_client_define_macro (MilterClientContext *context, MilterCommand command,
     gchar *inspected_macros;
 
     inspected_macros = milter_utils_inspect_hash_string_string(macros);
-    milter_debug("[manager][receive][define-macro] <%s>",
+    milter_debug("[%u] [manager][receive][define-macro] <%s>",
+                 milter_agent_get_tag(MILTER_AGENT(context)),
                  inspected_macros);
     g_free(inspected_macros);
 
@@ -357,7 +395,8 @@ cb_client_timeout (MilterClientContext *context, gpointer user_data)
 {
     MilterManagerLeader *leader = user_data;
 
-    milter_debug("[manager][timeout]");
+    milter_debug("[%u] [manager][timeout]",
+                 milter_agent_get_tag(MILTER_AGENT(context)));
 
     milter_manager_leader_timeout(leader);
 }
@@ -366,20 +405,55 @@ static void
 cb_client_finished (MilterClientContext *context, gpointer user_data)
 {
     MilterManagerLeader *leader = user_data;
+    GTimer *timer;
+    gdouble elapsed;
+    MilterClientContextState state, last_state;
+    MilterStatus status;
+    gchar *state_name, *last_state_name;
+    gchar *status_name, *statistics_status_name;
+    guint tag;
 
-    milter_debug("[manager][finish]");
+    timer = milter_client_context_get_private_data(context);
+    g_timer_stop(timer);
+    elapsed = g_timer_elapsed(timer, NULL);
+
+
+    state = milter_client_context_get_state(context);
+    state_name =
+        milter_utils_get_enum_nick_name(MILTER_TYPE_CLIENT_CONTEXT_STATE,
+                                        state);
+    last_state = milter_client_context_get_last_state(context);
+    last_state_name =
+        milter_utils_get_enum_nick_name(MILTER_TYPE_CLIENT_CONTEXT_STATE,
+                                        last_state);
+    status = milter_client_context_get_status(context);
+    status_name = milter_utils_get_enum_nick_name(MILTER_TYPE_STATUS,
+                                                  status);
+    tag = milter_agent_get_tag(MILTER_AGENT(context));
+    milter_debug("[%u] [manager][finish][%s][%s][%g]",
+                 tag, state_name, status_name, elapsed);
+    if (MILTER_STATUS_IS_PASS(status)) {
+        statistics_status_name = "pass";
+    } else {
+        statistics_status_name = status_name;
+    }
+    milter_statistics("[session][end][%s][%s][%g](%u)",
+                      last_state_name, statistics_status_name, elapsed, tag);
+    g_free(status_name);
+    g_free(state_name);
+    g_free(last_state_name);
 
     milter_manager_leader_quit(leader);
 }
 
 static void
 teardown_client_context_signals (MilterClientContext *context,
-                                 gpointer user_data)
+                                 MilterManagerLeader *leader)
 {
 #define DISCONNECT(name)                                                \
     g_signal_handlers_disconnect_by_func(context,                       \
                                          G_CALLBACK(cb_client_ ## name), \
-                                         user_data)
+                                         leader)
 
     DISCONNECT(negotiate);
     DISCONNECT(connect);
@@ -399,15 +473,28 @@ teardown_client_context_signals (MilterClientContext *context,
     DISCONNECT(finished);
 
 #undef DISCONNECT
-    g_signal_handlers_disconnect_by_func(user_data,
+    g_signal_handlers_disconnect_by_func(leader,
                                          G_CALLBACK(cb_leader_finished),
                                          context);
 }
 
 static gboolean
-cb_idle_unref (gpointer data)
+cb_idle_notify_finish_session_to_configuration (gpointer data)
 {
-    g_object_unref(data);
+    MilterManagerConfiguration *config = data;
+
+    milter_manager_configuration_session_finished(config);
+
+    return FALSE;
+}
+
+static gboolean
+cb_idle_unref_leader (gpointer data)
+{
+    MilterManagerLeader *leader = data;
+
+    g_object_unref(leader);
+
     return FALSE;
 }
 
@@ -417,15 +504,17 @@ cb_leader_finished (MilterFinishedEmittable *emittable, gpointer user_data)
     MilterClientContext *client_context = user_data;
     GTimer *timer;
     MilterManagerLeader *leader;
+    MilterManagerConfiguration *config;
 
     timer = milter_client_context_get_private_data(client_context);
     g_timer_stop(timer);
+
     leader = MILTER_MANAGER_LEADER(emittable);
-    teardown_client_context_signals(client_context, emittable);
-    milter_statistics("[session][end][%g](%p)",
-                      g_timer_elapsed(timer, NULL),
-                      client_context);
-    g_idle_add(cb_idle_unref, emittable);
+    teardown_client_context_signals(client_context, leader);
+
+    config = milter_manager_leader_get_configuration(leader);
+    g_idle_add(cb_idle_notify_finish_session_to_configuration, config);
+    g_idle_add(cb_idle_unref_leader, leader);
 }
 
 static void
@@ -473,13 +562,17 @@ static void
 connection_established (MilterClient *client, MilterClientContext *context)
 {
     GTimer *timer;
+    guint tag;
 
     setup_context_signals(context, MILTER_MANAGER(client));
 
     timer = g_timer_new();
     milter_client_context_set_private_data(context, timer,
                                            (GDestroyNotify)g_timer_destroy);
-    milter_statistics("[session][start](%p)", context);
+
+    tag = milter_agent_get_tag(MILTER_AGENT(context));
+    milter_debug("[%u] [manager][session][start]", tag),
+    milter_statistics("[session][start](%u)", tag);
 }
 
 static gchar *
@@ -544,6 +637,33 @@ is_remove_unix_socket_on_close (MilterClient *client)
     return milter_manager_configuration_is_remove_manager_unix_socket_on_close(configuration);
 }
 
+static guint
+get_suspend_time_on_unacceptable (MilterClient *client)
+{
+    MilterManager *manager;
+    MilterManagerPrivate *priv;
+    MilterManagerConfiguration *configuration;
+
+    manager = MILTER_MANAGER(client);
+    priv = MILTER_MANAGER_GET_PRIVATE(manager);
+    configuration = priv->configuration;
+    return milter_manager_configuration_get_suspend_time_on_unacceptable(configuration);
+}
+
+static void
+set_suspend_time_on_unacceptable (MilterClient *client, guint suspend_time)
+{
+    MilterManager *manager;
+    MilterManagerPrivate *priv;
+    MilterManagerConfiguration *configuration;
+
+    manager = MILTER_MANAGER(client);
+    priv = MILTER_MANAGER_GET_PRIVATE(manager);
+    configuration = priv->configuration;
+    milter_manager_configuration_set_suspend_time_on_unacceptable(configuration,
+                                                                  suspend_time);
+}
+
 MilterManagerConfiguration *
 milter_manager_get_configuration (MilterManager *manager)
 {
