@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -316,6 +316,17 @@ gboolean             milter_server_context_establish_connection
  */
 MilterStatus         milter_server_context_get_status  (MilterServerContext *context);
 
+/**
+ * milter_server_context_set_status:
+ * @context: a %MilterServerContext.
+ * @status: the new status.
+ *
+ * Sets the current status.
+ */
+void                 milter_server_context_set_status  (MilterServerContext *context,
+                                                        MilterStatus         status);
+
+
 /**
  * milter_server_context_get_state:
  * @context: a %MilterServerContext.
@@ -336,6 +347,18 @@ MilterServerContextState milter_server_context_get_state (MilterServerContext *c
 void                 milter_server_context_set_state   (MilterServerContext *context,
                                                         MilterServerContextState state);
 
+/**
+ * milter_server_context_get_last_state:
+ * @context: a %MilterServerContext.
+ *
+ * Gets the last state. It's one of start, negotiate,
+ * connect, helo, envelope-from, envelope-recipient, data,
+ * unknown, header, end-of-header, body and end-of-message.
+ *
+ * Returns: the last state.
+ */
+MilterServerContextState milter_server_context_get_last_state (MilterServerContext *context);
+
 /**
  * milter_server_context_is_processing:
  * @context: a %MilterServerContext.
@@ -511,6 +534,24 @@ gboolean             milter_server_context_quit        (MilterServerContext *con
  */
 gboolean             milter_server_context_abort       (MilterServerContext *context);
 
+/**
+ * milter_server_context_set_option:
+ * @context: a %MilterServerContext.
+ * @option: the negotiate option.
+ *
+ * Set negotiate option.
+ */
+void                 milter_server_context_set_option  (MilterServerContext *context,
+                                                        MilterOption        *option);
+
+/**
+ * milter_server_context_get_option:
+ * @context: a %MilterServerContext.
+ *
+ * Get negotiate option.
+ */
+MilterOption        *milter_server_context_get_option  (MilterServerContext *context);
+
 /**
  * milter_server_context_is_enable_step:
  * @context: a %MilterServerContext.
@@ -557,6 +598,74 @@ const gchar         *milter_server_context_get_name    (MilterServerContext *con
  */
 void                 milter_server_context_set_name    (MilterServerContext *context,
                                                         const gchar *name);
+
+/**
+ * milter_server_context_get_elapsed:
+ * @context: a %MilterServerContext.
+ *
+ * Gets the elapsed time.
+ */
+gdouble              milter_server_context_get_elapsed (MilterServerContext *context);
+
+/**
+ * milter_server_context_is_negotiated:
+ * @context: a %MilterServerContext.
+ *
+ * Gets whether negotiation is succeeded.
+ */
+gboolean             milter_server_context_is_negotiated
+                                                       (MilterServerContext *context);
+
+/**
+ * milter_server_context_is_processing_message:
+ * @context: a %MilterServerContext.
+ *
+ * Gets whether the context is processing message.
+ */
+gboolean             milter_server_context_is_processing_message
+                                                       (MilterServerContext *context);
+
+/**
+ * milter_server_context_set_processing_message:
+ * @context: a %MilterServerContext.
+ * @processing: whether the context is processing message.
+ *
+ * Sets whether the context is processing message.
+ */
+void                 milter_server_context_set_processing_message
+                                                       (MilterServerContext *context,
+                                                        gboolean processing_message);
+
+/**
+ * milter_server_context_is_quitted:
+ * @context: a %MilterServerContext.
+ *
+ * Gets whether the context is quitted.
+ */
+gboolean             milter_server_context_is_quitted  (MilterServerContext *context);
+
+/**
+ * milter_server_context_set_quitted:
+ * @context: a %MilterServerContext.
+ * @quitted: whether the context is quitted.
+ *
+ * Sets whether the context is quitted.
+ */
+void                 milter_server_context_set_quitted (MilterServerContext *context,
+                                                        gboolean quitted);
+
+/**
+ * milter_server_context_reset_message_related_data:
+ * @context: a %MilterServerContext.
+ *
+ * Resets message related data in the context. It should be
+ * called before each 2nd or later messages in the same
+ * milter session. There are no bad effects if it is called
+ * before 1st message.
+ */
+void                 milter_server_context_reset_message_related_data
+                                                       (MilterServerContext *context);
+
 G_END_DECLS
 
 #endif /* __MILTER_SERVER_CONTEXT_H__ */
