@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -37,6 +37,7 @@ void test_encode_define_macro (gconstpointer data);
 void test_encode_connect_ipv4 (void);
 void test_encode_connect_ipv6 (void);
 void test_encode_connect_unix (void);
+void test_encode_connect_unknown (void);
 void test_encode_helo (void);
 void test_encode_envelope_from (void);
 void test_encode_envelope_recipient (void);
@@ -216,8 +217,8 @@ static void
 setup_define_macro_connect_packet (void)
 {
     g_string_append(expected, "C");
-    append_name_and_value(expected, "j", "debian.cozmixng.org");
-    append_name_and_value(expected, "{daemon_name}", "debian.cozmixng.org");
+    append_name_and_value(expected, "j", "debian.example.com");
+    append_name_and_value(expected, "{daemon_name}", "debian.example.com");
     append_name_and_value(expected, "v", "Postfix 2.5.5");
 }
 
@@ -231,14 +232,14 @@ static void
 setup_define_macro_envelope_from_packet (void)
 {
     g_string_append(expected, "M");
-    append_name_and_value(expected, "{mail_addr}", "kou@cozmixng.org");
+    append_name_and_value(expected, "{mail_addr}", "kou@example.com");
 }
 
 static void
 setup_define_macro_envelope_recipient_packet (void)
 {
     g_string_append(expected, "R");
-    append_name_and_value(expected, "{rcpt_addr}", "kou@cozmixng.org");
+    append_name_and_value(expected, "{rcpt_addr}", "kou@example.com");
 }
 
 static void
@@ -254,8 +255,8 @@ data_encode_define_macro (void)
     cut_add_data("connect",
                  define_macro_test_data_new(setup_define_macro_connect_packet,
                                             MILTER_COMMAND_CONNECT,
-                                            "j", "debian.cozmixng.org",
-                                            "daemon_name", "debian.cozmixng.org",
+                                            "j", "debian.example.com",
+                                            "daemon_name", "debian.example.com",
                                             "v", "Postfix 2.5.5",
                                             NULL),
                  define_macro_test_data_free);
@@ -270,7 +271,7 @@ data_encode_define_macro (void)
                  define_macro_test_data_new(
                      setup_define_macro_envelope_from_packet,
                      MILTER_COMMAND_ENVELOPE_FROM,
-                     "mail_addr", "kou@cozmixng.org",
+                     "mail_addr", "kou@example.com",
                      NULL),
                  define_macro_test_data_free);
 
@@ -278,7 +279,7 @@ data_encode_define_macro (void)
                  define_macro_test_data_new(
                      setup_define_macro_envelope_recipient_packet,
                      MILTER_COMMAND_ENVELOPE_RECIPIENT,
-                     "rcpt_addr", "kou@cozmixng.org",
+                     "rcpt_addr", "kou@example.com",
                      NULL),
                  define_macro_test_data_free);
 
@@ -411,6 +412,29 @@ test_encode_connect_unix (void)
     cut_assert_equal_memory(expected->str, expected->len, actual, actual_size);
 }
 
+void
+test_encode_connect_unknown (void)
+{
+    struct sockaddr address;
+    const gchar host_name[] = "unknown";
+    gsize actual_size = 0;
+
+    g_string_append(expected, "C");
+    g_string_append(expected, host_name);
+    g_string_append_c(expected, '\0');
+    g_string_append(expected, "U");
+    pack(expected);
+
+    address.sa_family = AF_UNSPEC;
+
+    milter_command_encoder_encode_connect(encoder,
+                                          &actual, &actual_size,
+                                          host_name,
+                                          &address,
+                                          sizeof(address));
+    cut_assert_equal_memory(expected->str, expected->len, actual, actual_size);
+}
+
 void
 test_encode_helo (void)
 {
@@ -429,7 +453,7 @@ test_encode_helo (void)
 void
 test_encode_envelope_from (void)
 {
-    const gchar from[] = "<kou@cozmixng.org>";
+    const gchar from[] = "<kou@example.com>";
     gsize actual_size = 0;
 
     g_string_append(expected, "M");
@@ -445,7 +469,7 @@ test_encode_envelope_from (void)
 void
 test_encode_envelope_recipient (void)
 {
-    const gchar to[] = "<kou@cozmixng.org>";
+    const gchar to[] = "<kou@example.com>";
     gsize actual_size = 0;
 
     g_string_append(expected, "R");
@@ -474,7 +498,7 @@ test_encode_data (void)
 void
 test_encode_header (void)
 {
-    const gchar from[] = "<kou@cozmixng.org>";
+    const gchar from[] = "<kou@example.com>";
     gsize actual_size = 0;
 
     g_string_append(expected, "L");
