@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -50,6 +50,7 @@ void test_envelope_from (void);
 void test_envelope_recipient (void);
 void test_data (void);
 void test_header (void);
+void test_header_with_protocol_version2 (void);
 void test_end_of_header (void);
 void data_important_status (void);
 void test_important_status (gconstpointer data);
@@ -865,11 +866,13 @@ static void
 do_negotiate (MilterManagerTestScenario *scenario, const gchar *group)
 {
     MilterOption *option;
+    guint n_emitted;
 
     option = milter_manager_test_scenario_get_option(scenario, group, NULL);
     gcut_take_object(G_OBJECT(option));
     milter_manager_children_negotiate(children, option, NULL);
-    wait_reply(1, n_negotiate_reply_emitted);
+    n_emitted = get_integer(scenario, group, "n_emitted");
+    wait_reply(n_emitted, n_negotiate_reply_emitted);
     cut_trace(assert_response(scenario, group));
 
     milter_assert_equal_option(option, actual_option);
@@ -1039,6 +1042,7 @@ data_scenario (void)
                  "negotiate - retry - fail",
                  g_strdup("negotiate-retry-fail.txt"), g_free,
                  "connect", g_strdup("connect.txt"), g_free,
+                 "connect - fail", g_strdup("connect-fail.txt"), g_free,
                  /* WRITEME */
                  /* "connect - stop", g_strdup("connect-stop.txt"), g_free */
                  /* "connect - half stop", g_strdup("connect-half-stop.txt"),
@@ -1083,7 +1087,7 @@ test_scenario (gconstpointer data)
 void
 test_negotiate (void)
 {
-    option = milter_option_new(2,
+    option = milter_option_new(6,
                                MILTER_ACTION_ADD_HEADERS |
                                MILTER_ACTION_CHANGE_BODY,
                                MILTER_STEP_NONE);
@@ -1114,8 +1118,8 @@ test_connect_with_macro (void)
     inet_aton(ip_address, &(address.sin_addr));
 
     connect_macros =
-        gcut_hash_table_string_string_new("j", "debian.cozmixng.org",
-                                          "daemon_name", "debian.cozmixng.org",
+        gcut_hash_table_string_string_new("j", "debian.example.com",
+                                          "daemon_name", "debian.example.com",
                                           "v", "Postfix 2.5.5",
                                           NULL);
     milter_manager_children_define_macro(children,
@@ -1215,6 +1219,21 @@ test_connect_stop (void)
     wait_reply(1, n_accept_emitted);
 }
 
+static gboolean
+have_quitted_context (GList *child_list)
+{
+    GList *node;
+
+    for (node = child_list; node; node = g_list_next(node)) {
+        MilterServerContext *context;
+
+        context = MILTER_SERVER_CONTEXT(node->data);
+        if (milter_server_context_is_quitted(context))
+            return TRUE;
+    }
+    return FALSE;
+}
+
 void
 test_connect_half_stop (void)
 {
@@ -1232,13 +1251,13 @@ test_connect_half_stop (void)
     address.sin_port = g_htons(50443);
     inet_aton(ip_address, &(address.sin_addr));
 
-    cut_assert_null(milter_manager_children_get_quitted_children(children));
+    cut_assert_false(have_quitted_context(child_list));
     milter_manager_children_connect(children,
                                     host_name,
                                     (struct sockaddr *)(&address),
                                     sizeof(address));
     wait_reply(1, n_continue_emitted);
-    cut_assert_not_null(milter_manager_children_get_quitted_children(children));
+    cut_assert_true(have_quitted_context(child_list));
 }
 
 void
@@ -1295,6 +1314,22 @@ test_header (void)
     wait_reply(6, n_continue_emitted);
 }
 
+void
+test_header_with_protocol_version2 (void)
+{
+    const gchar name[] = "X-Test-Header";
+    const gchar value[] = "Test Header Value";
+
+    arguments_append(arguments1,
+                     "--negotiate-version", "2",
+                     NULL);
+
+    cut_trace(test_envelope_recipient());
+
+    milter_manager_children_header(children, name, value);
+    wait_reply(6, n_continue_emitted);
+}
+
 void
 test_end_of_header (void)
 {
@@ -1569,6 +1604,7 @@ void
 test_no_negotiation (void)
 {
     MilterLogger *logger;
+    MilterManagerChild *child10026, *child10027;
 
     logger = milter_logger();
     log_signal_id = g_signal_connect(logger, "log", G_CALLBACK(cb_log), NULL);
@@ -1581,17 +1617,27 @@ test_no_negotiation (void)
     add_child("milter@10026", "inet:10026@localhost");
     add_child("milter@10027", "inet:10027@localhost");
 
+    child10026 = milter_manager_children_get_children(children)->data;
+    child10027 = milter_manager_children_get_children(children)->next->data;
+
     cut_assert_true(milter_manager_children_negotiate(children, option, NULL));
     g_main_context_iteration(NULL, FALSE);
-    cut_assert_equal_string("[children][error][connection] "
-                            "Failed to connect to inet:10026@localhost: "
-                            "Connection refused: milter@10026\n"
-                            "[children][error][connection] "
-                            "Failed to connect to inet:10027@localhost: "
-                            "Connection refused: milter@10027\n"
-                            "[children][error][negotiate][no-response] "
-                            "milter@10027\n",
-                            error_message->str);
+    cut_assert_equal_string(
+        cut_take_printf("[%u] [children][error][connection] [%u] "
+                        "Failed to connect to inet:10026@localhost: "
+                        "Connection refused: milter@10026\n"
+                        "[%u] [children][error][connection] [%u] "
+                        "Failed to connect to inet:10027@localhost: "
+                        "Connection refused: milter@10027\n"
+                        "[%u] [children][error][negotiate][no-response] [%u] "
+                        "milter@10027\n",
+                        milter_manager_children_get_tag(children),
+                        milter_agent_get_tag(MILTER_AGENT(child10026)),
+                        milter_manager_children_get_tag(children),
+                        milter_agent_get_tag(MILTER_AGENT(child10027)),
+                        milter_manager_children_get_tag(children),
+                        milter_agent_get_tag(MILTER_AGENT(child10027))),
+        error_message->str);
 }
 
 static void
@@ -1628,6 +1674,7 @@ void
 test_end_of_message_timeout (void)
 {
     MilterLogger *logger;
+    MilterManagerChild *child;
 
     arguments_append(arguments1,
                      "--action", "no_response",
@@ -1640,16 +1687,20 @@ test_end_of_message_timeout (void)
     logger = milter_logger();
     log_signal_id = g_signal_connect(logger, "log", G_CALLBACK(cb_log), NULL);
 
-    prepare_timeout_test(milter_manager_children_get_children(children)->data,
-                         NULL);
+    child = milter_manager_children_get_children(children)->data;
+    prepare_timeout_test(child, NULL);
 
     milter_manager_children_end_of_message(children, "end", strlen("end"));
 
     wait_reply(7, n_continue_emitted);
     wait_reply(1, n_end_of_message_timeout_emitted);
 
-    cut_assert_equal_string("[children][timeout][end-of-message] milter@10026\n",
-                            error_message->str);
+    cut_assert_equal_string(
+        cut_take_printf("[%u] [children][timeout][end-of-message] [%u] "
+                        "milter@10026\n",
+                        milter_manager_children_get_tag(children),
+                        milter_agent_get_tag(MILTER_AGENT(child))),
+        error_message->str);
 }
 
 void
@@ -1693,8 +1744,11 @@ test_writing_timeout (void)
     wait_reply(1, n_writing_timeout_emitted);
     wait_reply(1, n_continue_emitted);
 
-    cut_assert_equal_string("[children][timeout][writing] milter@10027\n",
-                            error_message->str);
+    cut_assert_equal_string(
+        cut_take_printf("[%u] [children][timeout][writing] [%u] milter@10027\n",
+                        milter_manager_children_get_tag(children),
+                        milter_agent_get_tag(MILTER_AGENT(child))),
+        error_message->str);
 }
 
 void
@@ -1704,6 +1758,7 @@ test_reading_timeout (void)
     const gchar host_name[] = "mx.local.net";
     const gchar ip_address[] = "192.168.123.123";
     MilterLogger *logger;
+    MilterManagerChild *child;
 
     arguments_append(arguments2,
                      "--action", "no_response",
@@ -1716,8 +1771,8 @@ test_reading_timeout (void)
     logger = milter_logger();
     log_signal_id = g_signal_connect(logger, "log", G_CALLBACK(cb_log), NULL);
 
-    prepare_timeout_test(milter_manager_children_get_children(children)->next->data,
-                         NULL);
+    child = milter_manager_children_get_children(children)->next->data;
+    prepare_timeout_test(child, NULL);
 
     address.sin_family = AF_INET;
     address.sin_port = g_htons(50443);
@@ -1731,8 +1786,11 @@ test_reading_timeout (void)
     wait_reply(1, n_continue_emitted);
     wait_reply(1, n_reading_timeout_emitted);
 
-    cut_assert_equal_string("[children][timeout][reading] milter@10027\n",
-                            error_message->str);
+    cut_assert_equal_string(
+        cut_take_printf("[%u] [children][timeout][reading] [%u] milter@10027\n",
+                        milter_manager_children_get_tag(children),
+                        milter_agent_get_tag(MILTER_AGENT(child))),
+        error_message->str);
 }
 
 /*
