@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -40,6 +40,9 @@ void test_remove_manager_unix_socket_on_create (void);
 void test_remove_controller_unix_socket_on_create (void);
 void test_daemon (void);
 void test_pid_file (void);
+void test_maintenance_interval (void);
+void test_suspend_time_on_unacceptable (void);
+void test_custom_configuration_directory (void);
 void test_controller_connection_spec (void);
 void test_manager_connection_spec (void);
 void test_fallback_status (void);
@@ -53,6 +56,7 @@ void test_find_applicable_condition (void);
 void test_remove_applicable_condition (void);
 void test_clear (void);
 void test_load_paths (void);
+void test_load_absolute_path (void);
 void test_save_custom (void);
 void test_to_xml_full (void);
 void test_to_xml_signal (void);
@@ -330,6 +334,45 @@ test_pid_file (void)
                             milter_manager_configuration_get_pid_file(config));
 }
 
+void
+test_maintenance_interval (void)
+{
+    cut_assert_equal_uint(
+        100,
+        milter_manager_configuration_get_maintenance_interval(config));
+    milter_manager_configuration_set_maintenance_interval(config, 1000);
+    cut_assert_equal_uint(
+        1000,
+        milter_manager_configuration_get_maintenance_interval(config));
+}
+
+void
+test_suspend_time_on_unacceptable (void)
+{
+    cut_assert_equal_uint(
+        MILTER_CLIENT_DEFAULT_SUSPEND_TIME_ON_UNACCEPTABLE,
+        milter_manager_configuration_get_suspend_time_on_unacceptable(config));
+    milter_manager_configuration_set_suspend_time_on_unacceptable(config, 10);
+    cut_assert_equal_uint(
+        10,
+        milter_manager_configuration_get_suspend_time_on_unacceptable(config));
+}
+
+void
+test_custom_configuration_directory (void)
+{
+    const gchar custom_configuration_directory[] = "/tmp/milter-manager/";
+
+    cut_assert_equal_string(
+        NULL,
+        milter_manager_configuration_get_custom_configuration_directory(config));
+    milter_manager_configuration_set_custom_configuration_directory(
+        config, custom_configuration_directory);
+    cut_assert_equal_string(
+        custom_configuration_directory,
+        milter_manager_configuration_get_custom_configuration_directory(config));
+}
+
 void
 test_controller_connection_spec (void)
 {
@@ -437,6 +480,9 @@ milter_assert_default_configuration_helper (MilterManagerConfiguration *config)
     cut_assert_equal_string(
         NULL,
         milter_manager_configuration_get_pid_file(config));
+    cut_assert_equal_string(
+        NULL,
+        milter_manager_configuration_get_custom_configuration_directory(config));
 
     cut_assert_equal_uint(
         0660,
@@ -457,6 +503,13 @@ milter_assert_default_configuration_helper (MilterManagerConfiguration *config)
     cut_assert_true(milter_manager_configuration_is_remove_controller_unix_socket_on_create(config));
     cut_assert_false(milter_manager_configuration_is_daemon(config));
 
+    cut_assert_equal_uint(
+        100,
+        milter_manager_configuration_get_maintenance_interval(config));
+    cut_assert_equal_uint(
+        MILTER_CLIENT_DEFAULT_SUSPEND_TIME_ON_UNACCEPTABLE,
+        milter_manager_configuration_get_suspend_time_on_unacceptable(config));
+
     gcut_assert_equal_enum(
         MILTER_TYPE_STATUS,
         MILTER_STATUS_ACCEPT,
@@ -668,6 +721,9 @@ test_clear (void)
     test_controller_connection_spec();
     test_fallback_status();
     test_children();
+    test_daemon();
+    test_maintenance_interval();
+    test_suspend_time_on_unacceptable();
 
     milter_manager_configuration_clear(config);
     milter_assert_default_configuration(config);
@@ -709,6 +765,20 @@ test_load_paths (void)
         milter_manager_configuration_get_load_paths(config));
 }
 
+void
+test_load_absolute_path (void)
+{
+    const gchar *config_file;
+    GError *error = NULL;
+
+    config_file = cut_build_path(tmp_dir, "temporary.conf", NULL);
+    g_file_set_contents(config_file, "", 0, &error);
+    gcut_assert_error(error);
+
+    milter_manager_configuration_load(config, config_file, &error);
+    gcut_assert_error(error);
+}
+
 void
 test_save_custom (void)
 {
@@ -760,6 +830,7 @@ test_to_xml_full (void)
         "    <milter>\n"
         "      <name>milter@10025</name>\n"
         "      <enabled>true</enabled>\n"
+        "      <fallback-status>accept</fallback-status>\n"
         "      <connection-spec>inet:10025</connection-spec>\n"
         "      <command>test-milter</command>\n"
         "    </milter>\n"
@@ -797,6 +868,7 @@ test_to_xml_signal (void)
         "    <milter>\n"
         "      <name>milter@10025</name>\n"
         "      <enabled>true</enabled>\n"
+        "      <fallback-status>accept</fallback-status>\n"
         "      <connection-spec>inet:10025</connection-spec>\n"
         "      <command>test-milter</command>\n"
         "      <command-options>--user nobody</command-options>\n"
@@ -814,6 +886,7 @@ test_to_xml_signal (void)
         "    <milter>\n"
         "      <name>milter@10025</name>\n"
         "      <enabled>true</enabled>\n"
+        "      <fallback-status>accept</fallback-status>\n"
         "      <connection-spec>inet:10025</connection-spec>\n"
         "      <command>test-milter</command>\n"
         "      <command-options>--user nobody</command-options>\n"
