@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -37,6 +37,8 @@ void test_scenario (gconstpointer data);
 void data_large_body (void);
 void test_large_body (gconstpointer data);
 
+void test_configuration (void);
+
 static gchar *scenario_dir;
 static MilterManagerTestScenario *main_scenario;
 
@@ -603,10 +605,46 @@ assert_response_error (MilterManagerTestScenario *scenario, const gchar *group)
         cut_return();
 }
 
+#define assert_n_received(scenario, group, n_received_key, response_name) \
+    cut_trace_with_info_expression(                                     \
+        assert_n_received_helper(scenario, group,                       \
+                                 n_received_key, response_name),        \
+        assert_n_received(scenario, group, n_received_key, response_name))
+
+static void
+assert_n_received_helper (MilterManagerTestScenario *scenario,
+                          const gchar *group,
+                          const gchar *n_received_key,
+                          const gchar *response_name)
+{
+    MilterManagerTestClientGetNReceived get_n_received;
+    guint n_received, actual_n_received;
+
+    get_n_received = response_to_get_n_received(response_name);
+    cut_set_message("response: <%s>(%s)", response_name, group);
+    cut_assert_not_null(get_n_received);
+
+    n_received = get_integer(scenario, group, n_received_key);
+    if (n_received > g_list_length((GList *)started_clients)) {
+        milter_manager_test_clients_wait_reply(started_clients,
+                                               get_n_received);
+    } else if (n_received > 0) {
+        milter_manager_test_clients_wait_n_replies(started_clients,
+                                                   get_n_received,
+                                                   n_received);
+    }
+
+    actual_n_received =
+        milter_manager_test_clients_collect_n_received(started_clients,
+                                                       get_n_received);
+    cut_set_message("[%s]<%s>: <%s>=<%u>",
+                    group, response_name, n_received_key, n_received);
+    cut_assert_equal_uint(n_received, actual_n_received);
+}
+
 static void
 assert_response_common (MilterManagerTestScenario *scenario, const gchar *group)
 {
-    MilterManagerTestClientGetNReceived get_n_received;
     const gchar *response = NULL;
     MilterStatus status;
 
@@ -622,6 +660,7 @@ assert_response_common (MilterManagerTestScenario *scenario, const gchar *group)
             if (g_str_equal(response, "none")) {
                 assert_nothing_output(client);
             } else {
+                MilterManagerTestClientGetNReceived get_n_received;
                 get_n_received = response_to_get_n_received(response);
                 wait_reply_from_client(client, get_n_received);
             }
@@ -644,23 +683,14 @@ assert_response_common (MilterManagerTestScenario *scenario, const gchar *group)
     }
 
     if (!g_str_equal(response, "quit") && !g_str_equal(response, "none")) {
-        guint n_received, actual_n_received;
-
-        get_n_received = response_to_get_n_received(response);
-        cut_assert_not_null(get_n_received,
-                            "response: <%s>(%s)", response, group);
-
-        n_received = get_integer(scenario, group, "n_received");
-        if (n_received >= g_list_length((GList *)started_clients))
-            milter_manager_test_clients_wait_reply(started_clients,
-                                                   get_n_received);
-
-        actual_n_received =
-            milter_manager_test_clients_collect_n_received(started_clients,
-                                                           get_n_received);
-        cut_assert_equal_uint(n_received, actual_n_received, "[%s]", group);
+        assert_n_received(scenario, group, "n_received", response);
     }
 
+    if (has_key(scenario, group, "n_abort")) {
+        assert_n_received(scenario, group, "n_abort", "abort");
+    }
+
+    g_main_context_iteration(NULL, FALSE);
     gcut_assert_equal_enum(MILTER_TYPE_STATUS, status, response_status,
                            "[%s]", group);
 
@@ -678,14 +708,62 @@ assert_response_reply_code (MilterManagerTestScenario *scenario, const gchar *gr
         milter_manager_test_server_get_received_reply_codes(server));
 }
 
+#define assert_response(scenario, group)                \
+    cut_trace_with_info_expression(                     \
+        assert_response_helper(scenario, group),        \
+        assert_response(scenario, group))
+
 static void
-assert_response (MilterManagerTestScenario *scenario, const gchar *group)
+assert_response_helper (MilterManagerTestScenario *scenario, const gchar *group)
 {
     cut_trace(assert_response_error(scenario, group));
     cut_trace(assert_response_common(scenario, group));
     cut_trace(assert_response_reply_code(scenario, group));
 }
 
+#define assert_alive(scenario, group)           \
+    cut_trace_with_info_expression(             \
+        assert_alive_helper(scenario, group),   \
+        assert_alive(scenario, group))
+
+static void
+assert_alive_helper (MilterManagerTestScenario *scenario, const gchar *group)
+{
+    guint expected_n_alive, actual_n_alive;
+
+    if (!has_key(scenario, group, "n_alive"))
+        return;
+
+    expected_n_alive = get_integer(scenario, group, "n_alive");
+    milter_manager_test_clients_wait_n_alive(started_clients, expected_n_alive);
+    actual_n_alive =
+        milter_manager_test_clients_collect_n_alive(started_clients);
+    cut_assert_equal_uint(expected_n_alive, actual_n_alive);
+}
+
+#define assert_macros(scenario, group, command)         \
+    cut_trace_with_info_expression(                     \
+        assert_macros_helper(scenario, group, command), \
+        assert_macros(scenario, group, command))
+
+static void
+assert_macros_helper (MilterManagerTestScenario *scenario, const gchar *group,
+                      MilterCommand command)
+{
+    const GList *expected_macros;
+    const GList *actual_macros;
+
+    expected_macros = get_pair_list_without_sort(scenario, group, "macros");
+    if (expected_macros) {
+        actual_macros = collect_received_macros(command);
+        gcut_assert_equal_list(
+            expected_macros, actual_macros,
+            milter_manager_test_pair_equal,
+            (GCutInspectFunction)milter_manager_test_pair_inspect,
+            NULL);
+    }
+}
+
 static void
 do_negotiate (MilterManagerTestScenario *scenario, const gchar *group)
 {
@@ -699,7 +777,8 @@ do_negotiate (MilterManagerTestScenario *scenario, const gchar *group)
     gcut_take_object(G_OBJECT(option));
     milter_server_context_negotiate(MILTER_SERVER_CONTEXT(server), option);
     milter_manager_leader_negotiate(leader, option, NULL);
-    cut_trace(assert_response(scenario, group));
+    assert_response(scenario, group);
+    assert_alive(scenario, group);
 
     if (milter_manager_test_scenario_has_group(main_scenario, group) &&
         milter_manager_test_scenario_has_key(main_scenario, group, "options")) {
@@ -714,6 +793,17 @@ do_negotiate (MilterManagerTestScenario *scenario, const gchar *group)
         (GEqualFunc)milter_option_equal);
 }
 
+static void
+define_macro (MilterManagerTestScenario *scenario, const gchar *group,
+              MilterCommand command)
+{
+    const GHashTable *macros;
+
+    macros = get_hash_table(scenario, group, "macro");
+    if (macros)
+        milter_manager_leader_define_macro(leader, command, (GHashTable *)macros);
+}
+
 static void
 do_connect (MilterManagerTestScenario *scenario, const gchar *group)
 {
@@ -725,9 +815,6 @@ do_connect (MilterManagerTestScenario *scenario, const gchar *group)
     socklen_t address_size;
     const GList *expected_connect_infos;
     const GList *actual_connect_infos;
-    const GHashTable *macros;
-    const GList *expected_macros;
-    const GList *actual_macros;
 
     milter_server_context_set_state(MILTER_SERVER_CONTEXT(server),
                                     MILTER_SERVER_CONTEXT_STATE_CONNECT);
@@ -739,13 +826,12 @@ do_connect (MilterManagerTestScenario *scenario, const gchar *group)
                                  &error);
     gcut_assert_error(error);
 
-    macros = get_hash_table(scenario, group, "macro");
-    if (macros)
-        milter_manager_leader_define_macro(leader, MILTER_COMMAND_CONNECT, (GHashTable*)macros);
+    define_macro(scenario, group, MILTER_COMMAND_CONNECT);
     milter_manager_leader_connect(leader, host, address, address_size);
     g_free(address);
 
-    cut_trace(assert_response(scenario, group));
+    assert_response(scenario, group);
+    assert_alive(scenario, group);
 
     expected_connect_infos = get_pair_list(scenario, group, "infos");
     actual_connect_infos = collect_received_pairs(connect_host, connect_address);
@@ -755,15 +841,7 @@ do_connect (MilterManagerTestScenario *scenario, const gchar *group)
         (GCutInspectFunction)milter_manager_test_pair_inspect,
         NULL);
 
-    expected_macros = get_pair_list_without_sort(scenario, group, "macros");
-    if (expected_macros) {
-        actual_macros = collect_received_macros(MILTER_COMMAND_CONNECT);
-        gcut_assert_equal_list(
-            expected_macros, actual_macros,
-            milter_manager_test_pair_equal,
-            (GCutInspectFunction)milter_manager_test_pair_inspect,
-            NULL);
-    }
+    assert_macros(scenario, group, MILTER_COMMAND_CONNECT);
 }
 
 static void
@@ -777,7 +855,8 @@ do_helo (MilterManagerTestScenario *scenario, const gchar *group)
     fqdn = get_string(scenario, group, "fqdn");
     milter_manager_leader_helo(leader, fqdn);
 
-    cut_trace(assert_response(scenario, group));
+    assert_response(scenario, group);
+    assert_alive(scenario, group);
 
     expected_fqdns = get_string_g_list(scenario, group, "fqdns");
     gcut_assert_equal_list_string(expected_fqdns,
@@ -794,14 +873,18 @@ do_envelope_from (MilterManagerTestScenario *scenario, const gchar *group)
     milter_server_context_set_state(MILTER_SERVER_CONTEXT(server),
                                     MILTER_SERVER_CONTEXT_STATE_ENVELOPE_FROM);
     from = get_string(scenario, group, "from");
+    define_macro(scenario, group, MILTER_COMMAND_ENVELOPE_FROM);
     milter_manager_leader_envelope_from(leader, from);
 
-    cut_trace(assert_response(scenario, group));
+    assert_response(scenario, group);
+    assert_alive(scenario, group);
 
     expected_froms = get_string_g_list(scenario, group, "froms");
     gcut_assert_equal_list_string(expected_froms,
                                   collect_received_strings(envelope_from),
                                   "[%s]", group);
+
+    assert_macros(scenario, group, MILTER_COMMAND_ENVELOPE_FROM);
 }
 
 static void
@@ -816,7 +899,8 @@ do_envelope_recipient (MilterManagerTestScenario *scenario, const gchar *group)
     n_envelope_recipient_responses = 0;
     milter_manager_leader_envelope_recipient(leader, recipient);
 
-    cut_trace(assert_response(scenario, group));
+    assert_response(scenario, group);
+    assert_alive(scenario, group);
 
     expected_recipients = get_string_g_list(scenario, group, "recipients");
     gcut_assert_equal_list_string(expected_recipients,
@@ -831,7 +915,7 @@ do_data (MilterManagerTestScenario *scenario, const gchar *group)
                                     MILTER_SERVER_CONTEXT_STATE_DATA);
     milter_manager_leader_data(leader);
 
-    cut_trace(assert_response(scenario, group));
+    assert_response(scenario, group);
 }
 
 static void
@@ -850,7 +934,8 @@ do_header (MilterManagerTestScenario *scenario, const gchar *group)
 
     milter_manager_test_server_add_header(server, name, value);
 
-    cut_trace(assert_response(scenario, group));
+    assert_response(scenario, group);
+    assert_alive(scenario, group);
 
     expected_headers = get_pair_list(scenario, group, "headers");
     actual_headers =
@@ -870,48 +955,113 @@ do_end_of_header (MilterManagerTestScenario *scenario, const gchar *group)
                                     MILTER_SERVER_CONTEXT_STATE_END_OF_HEADER);
     milter_manager_leader_end_of_header(leader);
 
-    cut_trace(assert_response(scenario, group));
+    assert_response(scenario, group);
+    assert_alive(scenario, group);
+}
+
+static void
+get_chunk (MilterManagerTestScenario *scenario, const gchar *group,
+           const gchar **chunk, gsize *chunk_size)
+{
+    *chunk = NULL;
+    *chunk_size = 0;
+
+    if (has_key(scenario, group, "chunk_file")) {
+        const gchar *file_name;
+        gchar *file_path;
+        gchar *content;
+        GError *error = NULL;
+
+        file_name = get_string(scenario, group, "chunk_file");
+        file_path = g_build_filename(scenario_dir, file_name, NULL);
+        g_file_get_contents(file_path, &content, chunk_size, &error);
+        if (content && content[*chunk_size - 1] == '\n')
+            content[*chunk_size - 1] = '\0';
+        g_free(file_path);
+        gcut_assert_error(error);
+        *chunk = cut_take_string(content);
+    } else if (has_key(scenario, group, "chunk")) {
+        *chunk = get_string(scenario, group, "chunk");
+        *chunk_size = strlen(*chunk);
+    }
+}
+
+static const GList *
+get_chunks (MilterManagerTestScenario *scenario, const gchar *group)
+{
+    if (has_key(scenario, group, "chunk_files")) {
+        const GList *file_names, *node, *taken_chunks;
+        GList *chunks = NULL;
+        GError *error = NULL;
+
+        file_names = get_string_g_list(scenario, group, "chunk_files");
+        for (node = file_names; node; node = g_list_next(node)) {
+            const gchar *file_name = node->data;
+            gchar *content = NULL;
+            gsize content_size = 0;
+
+            if (file_name) {
+                gchar *file_path;
+
+                file_path = g_build_filename(scenario_dir, file_name, NULL);
+                g_file_get_contents(file_path, &content, &content_size, &error);
+                g_free(file_path);
+                if (error)
+                    break;
+            }
+            if (content && content[content_size - 1] == '\n')
+                content[content_size - 1] = '\0';
+            chunks = g_list_append(chunks, content);
+        }
+        taken_chunks = gcut_take_list(chunks, g_free);
+        gcut_assert_error(error);
+        return taken_chunks;
+    } else {
+        return get_string_g_list(scenario, group, "chunks");
+    }
 }
 
 static void
 do_body (MilterManagerTestScenario *scenario, const gchar *group)
 {
-    const gchar *chunk;
+    const gchar *chunk = NULL;
+    gsize chunk_size = 0;
+    const GList *expected_chunks, *actual_chunks;
 
     milter_server_context_set_state(MILTER_SERVER_CONTEXT(server),
                                     MILTER_SERVER_CONTEXT_STATE_BODY);
-    chunk = get_string(scenario, group, "chunk");
+    get_chunk(scenario, group, &chunk, &chunk_size);
     n_body_responses = 0;
-    milter_manager_leader_body(leader, chunk, strlen(chunk));
+    milter_manager_leader_body(leader, chunk, chunk_size);
 
-    cut_trace(assert_response(scenario, group));
+    assert_response(scenario, group);
+    assert_alive(scenario, group);
 
-    gcut_assert_equal_list_string(get_string_g_list(scenario, group, "chunks"),
-                                  collect_received_strings(body_chunk),
-                                  "<%s>", group);
+    expected_chunks = get_chunks(scenario, group);
+    actual_chunks = collect_received_strings(body_chunk);
+    cut_set_message("[%s]", group);
+    gcut_assert_equal_list_string(expected_chunks, actual_chunks);
 }
 
 static void
 do_end_of_message (MilterManagerTestScenario *scenario, const gchar *group)
 {
     const gchar *chunk = NULL;
+    gsize chunk_size = 0;
 
     milter_server_context_set_state(MILTER_SERVER_CONTEXT(server),
                                     MILTER_SERVER_CONTEXT_STATE_END_OF_MESSAGE);
-    if (has_key(scenario, group, "chunk"))
-        chunk = get_string(scenario, group, "chunk");
+    get_chunk(scenario, group, &chunk, &chunk_size);
 
     milter_client_context_set_state(client_context,
                                     MILTER_CLIENT_CONTEXT_STATE_END_OF_MESSAGE);
-    milter_manager_leader_end_of_message(leader,
-                                         chunk,
-                                         chunk ? strlen(chunk) : 0);
+    milter_manager_leader_end_of_message(leader, chunk, chunk_size);
 
-    cut_trace(assert_response(scenario, group));
+    assert_response(scenario, group);
+    assert_alive(scenario, group);
 
-    gcut_assert_equal_list_string(
-        get_string_g_list(scenario, group, "chunks"),
-        collect_received_strings(body_chunk));
+    gcut_assert_equal_list_string(get_chunks(scenario, group),
+                                  collect_received_strings(body_chunk));
     gcut_assert_equal_list_string(
         get_string_g_list(scenario, group, "end_of_message_chunks"),
         collect_received_strings(end_of_message_chunk));
@@ -1059,14 +1209,14 @@ static void
 do_quit (MilterManagerTestScenario *scenario, const gchar *group)
 {
     milter_manager_leader_quit(leader);
-    cut_trace(assert_response(scenario, group));
+    assert_response(scenario, group);
 }
 
 static void
 do_abort (MilterManagerTestScenario *scenario, const gchar *group)
 {
     milter_manager_leader_abort(leader);
-    cut_trace(assert_response(scenario, group));
+    assert_response(scenario, group);
 }
 
 static void
@@ -1077,7 +1227,7 @@ do_unknown (MilterManagerTestScenario *scenario, const gchar *group)
     command = get_string(scenario, group, "unknown_command");
 
     milter_manager_leader_unknown(leader, command);
-    cut_trace(assert_response(scenario, group));
+    assert_response(scenario, group);
 
     gcut_assert_equal_list_string(get_string_g_list(scenario, group,
                                                     "unknown_commands"),
@@ -1094,43 +1244,43 @@ do_action (MilterManagerTestScenario *scenario, const gchar *group)
     case MILTER_COMMAND_NEGOTIATE:
         cut_trace(do_negotiate(scenario, group));
         break;
-      case MILTER_COMMAND_CONNECT:
+    case MILTER_COMMAND_CONNECT:
         cut_trace(do_connect(scenario, group));
         break;
-      case MILTER_COMMAND_HELO:
+    case MILTER_COMMAND_HELO:
         cut_trace(do_helo(scenario, group));
         break;
-      case MILTER_COMMAND_ENVELOPE_FROM:
+    case MILTER_COMMAND_ENVELOPE_FROM:
         cut_trace(do_envelope_from(scenario, group));
         break;
-      case MILTER_COMMAND_ENVELOPE_RECIPIENT:
+    case MILTER_COMMAND_ENVELOPE_RECIPIENT:
         cut_trace(do_envelope_recipient(scenario, group));
         break;
-      case MILTER_COMMAND_DATA:
+    case MILTER_COMMAND_DATA:
         cut_trace(do_data(scenario, group));
         break;
-      case MILTER_COMMAND_HEADER:
+    case MILTER_COMMAND_HEADER:
         cut_trace(do_header(scenario, group));
         break;
-      case MILTER_COMMAND_END_OF_HEADER:
+    case MILTER_COMMAND_END_OF_HEADER:
         cut_trace(do_end_of_header(scenario, group));
         break;
-      case MILTER_COMMAND_BODY:
+    case MILTER_COMMAND_BODY:
         cut_trace(do_body(scenario, group));
         break;
-      case MILTER_COMMAND_END_OF_MESSAGE:
+    case MILTER_COMMAND_END_OF_MESSAGE:
         cut_trace(do_end_of_message_full(scenario, group));
         break;
-      case MILTER_COMMAND_QUIT:
+    case MILTER_COMMAND_QUIT:
         cut_trace(do_quit(scenario, group));
         break;
-      case MILTER_COMMAND_ABORT:
+    case MILTER_COMMAND_ABORT:
         cut_trace(do_abort(scenario, group));
         break;
-      case MILTER_COMMAND_UNKNOWN:
+    case MILTER_COMMAND_UNKNOWN:
         cut_trace(do_unknown(scenario, group));
         break;
-      default:
+    default:
         cut_error("unknown command: <%c>(<%s>)", command, group);
         break;
     }
@@ -1155,6 +1305,8 @@ do_actions (MilterManagerTestScenario *scenario)
                               "actions", &length);
     for (i = 0; i < length; i++) {
         cut_trace(do_action(scenario, actions[i]));
+        cut_assert_equal_string(
+            NULL, milter_manager_test_server_get_last_error(server));
     }
 }
 
@@ -1167,59 +1319,28 @@ data_scenario_basic (void)
                  g_strdup("negotiate-failure.txt"), g_free,
                  "connect",
                  g_strdup("connect.txt"), g_free,
-                 "connect-with-macro",
+                 "connect - macro",
                  g_strdup("connect-with-macro.txt"), g_free,
                  "helo",
                  g_strdup("helo.txt"), g_free,
                  "envelope-from",
                  g_strdup("envelope-from.txt"), g_free,
+                 "envelope-from - macro",
+                 g_strdup("envelope-from-with-macro.txt"), g_free,
                  "envelope-recipient",
                  g_strdup("envelope-recipient.txt"), g_free,
                  "data",
                  g_strdup("data.txt"), g_free,
-                 "data - no data flag on second client",
-                 g_strdup("no-data-second-client.txt"), g_free,
-                 "data - close without reply",
-                 g_strdup("data-close-without-reply.txt"), g_free,
                  "header - from",
                  g_strdup("header-from.txt"), g_free,
                  "header - from-and-mailer",
                  g_strdup("header-from-and-mailer.txt"), g_free,
-                 "header - with-leading-space",
-                 g_strdup("header-with-leading-space.txt"), g_free,
-                 "header - with-leading-space - no space",
-                 g_strdup("header-with-leading-space-no-space.txt"), g_free,
                  "end-of-header",
                  g_strdup("end-of-header.txt"), g_free,
                  "body",
                  g_strdup("body.txt"), g_free,
-                 "body - larger than milter_chunk_size",
-                 g_strdup("body-larger-than-milter-chunk-size.txt"), g_free,
-                 "body - no body flag on both client",
-                 g_strdup("no-body-flag-on-both-client.txt"), g_free,
-                 "end-of-message - no body flag",
-                 g_strdup("no-body-flag.txt"), g_free,
-                 "end-of-message - no body flag on second client",
-                 g_strdup("no-body-flag-on-second-client.txt"), g_free,
                  "end-of-message",
                  g_strdup("end-of-message.txt"), g_free,
-                 "end-of-message - chunk",
-                 g_strdup("end-of-message-chunk.txt"), g_free,
-                 "end-of-message - reject",
-                 g_strdup("end-of-message-reject.txt"), g_free,
-                 "end-of-message - discard",
-                 g_strdup("end-of-message-discard.txt"), g_free,
-                 "end-of-message - temporary-failure",
-                 g_strdup("end-of-message-temporary-failure.txt"), g_free,
-                 "end-of-message - accept",
-                 g_strdup("end-of-message-accept.txt"), g_free,
-                 "end-of-message - replaced",
-                 g_strdup("end-of-message-replaced.txt"), g_free,
-                 "end-of-message - reply code - reject",
-                 g_strdup("end-of-message-reply-code-reject.txt"), g_free,
-                 "end-of-message - reply code - temporary failure",
-                 g_strdup("end-of-message-reply-code-temporary-failure.txt"),
-                 g_free,
                  "quit",
                  g_strdup("quit.txt"), g_free,
                  "abort - on envelope-from",
@@ -1236,6 +1357,8 @@ static void
 data_scenario_end_of_message_action (void)
 {
     cut_add_data("quarantine", g_strdup("quarantine.txt"), g_free,
+                 "quarantine - header-change",
+                 g_strdup("quarantine-header-change.txt"), g_free,
                  "add-header", g_strdup("add-header.txt"), g_free,
                  "add-header - with-leading-space",
                  g_strdup("add-header-with-leading-space.txt"), g_free,
@@ -1260,6 +1383,8 @@ static void
 data_scenario_envelope_from (void)
 {
     cut_add_data("envelope-from - reject",
+                 g_strdup("envelope-from-reject.txt"), g_free,
+                 "envelope-from - reject",
                  g_strdup("envelope-from-reject.txt"), g_free,
                  "envelope-from - discard",
                  g_strdup("envelope-from-discard.txt"), g_free,
@@ -1270,7 +1395,13 @@ data_scenario_envelope_from (void)
                  "envelope-from - accept",
                  g_strdup("envelope-from-accept.txt"), g_free,
                  "envelope-from - accept all",
-                 g_strdup("envelope-from-accept-all.txt"), g_free);
+                 g_strdup("envelope-from-accept-all.txt"), g_free,
+                 "envelope-from - stop",
+                 g_strdup("envelope-from-stop.txt"), g_free,
+                 "envelope-from - again",
+                 g_strdup("envelope-from-again.txt"), g_free,
+                 "envelope-from - discarded - again",
+                 g_strdup("envelope-from-discarded-again.txt"), g_free);
 }
 
 static void
@@ -1295,15 +1426,102 @@ data_scenario_envelope_recipient (void)
         "envelope-recipient - temporary-failure",
         g_strdup("envelope-recipient-temporary-failure.txt"), g_free,
         "envelope-recipient - temporary-failure - all",
-        g_strdup("envelope-recipient-temporary-failure-all.txt"), g_free);
+        g_strdup("envelope-recipient-temporary-failure-all.txt"), g_free,
+        "envelope-recipient - stop",
+        g_strdup("envelope-recipient-stop.txt"), g_free,
+        "envelope-recipient - again",
+        g_strdup("envelope-recipient-again.txt"), g_free);
+}
+
+static void
+data_scenario_data (void)
+{
+    cut_add_data("data - no data flag on second client",
+                 g_strdup("no-data-second-client.txt"), g_free,
+                 "data - close without reply",
+                 g_strdup("data-close-without-reply.txt"), g_free,
+                 "data - reply code reject",
+                 g_strdup("data-reply-code-reject.txt"), g_free,
+                 NULL);
+}
+
+static void
+data_scenario_header (void)
+{
+    cut_add_data("header - with-leading-space",
+                 g_strdup("header-with-leading-space.txt"), g_free,
+                 "header - with-leading-space - no space",
+                 g_strdup("header-with-leading-space-no-space.txt"), g_free,
+                 "header - temporary failure",
+                 g_strdup("header-temporary-failure.txt"), g_free,
+                 "header - reject",
+                 g_strdup("header-reject.txt"), g_free,
+                 "header - stop",
+                 g_strdup("header-stop.txt"), g_free,
+                 "header - stop - all",
+                 g_strdup("header-stop-all.txt"), g_free,
+                 "header - again",
+                 g_strdup("header-again.txt"), g_free,
+                 NULL);
+}
+
+static void
+data_scenario_end_of_header (void)
+{
+    cut_add_data("end-of-header - again",
+                 g_strdup("end-of-header-again.txt"), g_free,
+                 NULL);
 }
 
 static void
 data_scenario_body (void)
 {
-    cut_add_data("body - skip", g_strdup("body-skip.txt"), g_free,
-                 "body - skip all", g_strdup("body-skip-all.txt"), g_free,
-                 "body - accept", g_strdup("body-accept.txt"), g_free);
+    cut_add_data("body - larger than milter chunk size",
+                 g_strdup("body-larger-than-milter-chunk-size.txt"), g_free,
+                 "body - no body flag on both client",
+                 g_strdup("no-body-flag-on-both-client.txt"), g_free,
+                 "body - skip",
+                 g_strdup("body-skip.txt"), g_free,
+                 "body - skip all",
+                 g_strdup("body-skip-all.txt"), g_free,
+                 "body - accept",
+                 g_strdup("body-accept.txt"), g_free,
+                 "body - again",
+                 g_strdup("body-again.txt"), g_free,
+                 NULL);
+}
+
+static void
+data_scenario_end_of_message (void)
+{
+    cut_add_data("end-of-message - no body flag",
+                 g_strdup("no-body-flag.txt"), g_free,
+                 "end-of-message - no body flag on second client",
+                 g_strdup("no-body-flag-on-second-client.txt"), g_free,
+                 "end-of-message - chunk",
+                 g_strdup("end-of-message-chunk.txt"), g_free,
+                 "end-of-message - reject",
+                 g_strdup("end-of-message-reject.txt"), g_free,
+                 "end-of-message - discard",
+                 g_strdup("end-of-message-discard.txt"), g_free,
+                 "end-of-message - discard - again",
+                 g_strdup("end-of-message-discard-again.txt"), g_free,
+                 "end-of-message - temporary-failure",
+                 g_strdup("end-of-message-temporary-failure.txt"), g_free,
+                 "end-of-message - accept",
+                 g_strdup("end-of-message-accept.txt"), g_free,
+                 "end-of-message - replaced",
+                 g_strdup("end-of-message-replaced.txt"), g_free,
+                 "end-of-message - reply code - reject",
+                 g_strdup("end-of-message-reply-code-reject.txt"), g_free,
+                 "end-of-message - reply code - temporary failure",
+                 g_strdup("end-of-message-reply-code-temporary-failure.txt"),
+                 g_free,
+                 "end-of-message - again",
+                 g_strdup("end-of-message-again.txt"), g_free,
+                 "end-of-message - without header",
+                 g_strdup("end-of-message-without-header.txt"), g_free,
+                 NULL);
 }
 
 static void
@@ -1320,7 +1538,11 @@ data_scenario (void)
     data_scenario_end_of_message_action();
     data_scenario_envelope_from();
     data_scenario_envelope_recipient();
+    data_scenario_data();
+    data_scenario_header();
+    data_scenario_end_of_header();
     data_scenario_body();
+    data_scenario_end_of_message();
     data_scenario_reply_code();
 }
 
@@ -1329,6 +1551,7 @@ test_scenario (gconstpointer data)
 {
     const gchar *scenario_name = data;
     const gchar omit_key[] = "omit";
+    const gchar configuration_key[] = "configuration";
 
     main_scenario = milter_manager_test_scenario_new();
     cut_trace(milter_manager_test_scenario_load(main_scenario,
@@ -1341,6 +1564,21 @@ test_scenario (gconstpointer data)
                                   MILTER_MANAGER_TEST_SCENARIO_GROUP_NAME,
                                   omit_key));
 
+    if (has_key(main_scenario,
+                MILTER_MANAGER_TEST_SCENARIO_GROUP_NAME, configuration_key)) {
+        GError *error = NULL;
+        const gchar *config_file;
+
+        config_file =
+            cut_build_path(scenario_dir,
+                           get_string(main_scenario,
+                                      MILTER_MANAGER_TEST_SCENARIO_GROUP_NAME,
+                                      configuration_key),
+                           NULL);
+        milter_manager_configuration_load(config, config_file, &error);
+        gcut_assert_error(error);
+    }
+
     cut_trace(milter_manager_test_scenario_start_clients(main_scenario));
     cut_trace(do_actions(main_scenario));
 }
@@ -1372,6 +1610,13 @@ test_large_body (gconstpointer data)
     */
 }
 
+void
+test_configuration (void)
+{
+    gcut_assert_equal_object(config,
+                             milter_manager_leader_get_configuration(leader));
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
